﻿-- Corrige o Sequenciador para um valor correto caso nao esteja sincronizado
SELECT setval('sqlid', (select MAX(sqlid) +1 from SQL));


-- Adiciona funcoes utilizadas pela impressao

CREATE OR REPLACE FUNCTION ImpressaoEhTxt() RETURNS BOOLEAN AS $$
DECLARE
    ambiente_info RECORD;
BEGIN
    -- Obtem as informacoes do Ambiente 
    SELECT * INTO ambiente_info FROM Ambiente;

    -- Verifica se AmbienteImpressoraPorta contem "COM3" ou "USB" ou um caminho no sistema
    RETURN CASE
        WHEN POSITION('/' IN ambiente_info.AmbienteImpressoraPorta) > 0 OR POSITION('.' IN ambiente_info.AmbienteImpressoraPorta) > 0 OR ambiente_info.AmbienteImpressoraPorta = '' THEN
            TRUE
        ELSE
            FALSE
    END;
END;
$$ LANGUAGE plpgsql;


CREATE OR REPLACE FUNCTION adicionar_quebra_linha(apos_tag_acbr BOOLEAN DEFAULT FALSE) RETURNS VARCHAR AS $$
DECLARE
    ehTxt BOOLEAN;
    DesconsiderarTags BOOLEAN;
BEGIN
    -- Chama a função anterior para verificar se o texto contém uma porta ou caminho válido
    ehTxt := ImpressaoEhTxt();
    
    SELECT AmbienteImpressoraDesconsideraTags
      INTO DesconsiderarTags
      FROM Ambiente;
    
    -- Se é para desconsiderar tag, ignora parâmetro
    IF DesconsiderarTags THEN
      apos_tag_acbr := False;
    END IF;

    -- Se a quebra de linha for após uma tag do acbr, ignora pois já é adicionado o quebra linha automaticamente.
    IF apos_tag_acbr THEN
        IF ehTxt THEN
            RETURN '</lf>';
        ELSE
            RETURN '';
        END IF;
    ELSE
        RETURN '</lf>';
    END IF;
END;
$$ LANGUAGE plpgsql;

-- Função para concatenação dos textos
CREATE OR REPLACE FUNCTION completa_textos(texto1 TEXT, texto2 TEXT)
RETURNS TEXT AS $$
DECLARE
    tamanho_total CONSTANT INT := 46;
    espaco_interno CONSTANT CHAR := ' '; -- Caractere de separação
    total_caracteres INT;
    espacos_entre TEXT := ''; -- Inicializa a variável
    i INT;
BEGIN
    -- Calcula o total de caracteres
    total_caracteres := length(texto1) + length(texto2) + 2; -- +2 para os espaços

    -- Se o total de caracteres exceder o tamanho total, corta texto1 ou texto2
    IF total_caracteres > tamanho_total THEN
        RETURN substring(texto1 FROM 1 FOR tamanho_total - length(texto2) - 2) || ' ' || texto2;
    ELSE
        -- Calcula quantos caracteres de preenchimento são necessários
        IF espaco_interno = ' ' THEN  -- Adiciona 'THEN' aqui
            FOR i IN 1..(tamanho_total - total_caracteres) LOOP
                espacos_entre := espacos_entre || ' '; -- Concatena um espaço
            END LOOP;
        ELSE
            espacos_entre := repeat(espaco_interno, tamanho_total - total_caracteres);
        END IF;

        -- Monta o resultado final com espaço antes de texto2
        RETURN texto1 || ' ' || espacos_entre || ' ' || texto2;
    END IF;
END;
$$ LANGUAGE plpgsql;

-- Função para concatenação dos textos (versão para três expressôes)
CREATE OR REPLACE FUNCTION completa_textos_trio(texto1 TEXT, texto2 TEXT, texto3 TEXT)
RETURNS TEXT AS $$
DECLARE
    tamanho_total CONSTANT INT := 48;
    espaco_interno CONSTANT CHAR := ' '; -- Caractere de separação
    total_caracteres INT;
    espacos_entre1 TEXT := ''; -- Espaços entre texto1 e texto2
    espacos_entre2 TEXT := ''; -- Espaços entre texto2 e texto3
    i INT;
BEGIN
    -- Calcula o total de caracteres
    total_caracteres := length(texto1) + length(texto2) + length(texto3) + 4; -- +4 para os espaços entre os textos

    -- Se o total de caracteres exceder o tamanho total, corta os textos
    IF total_caracteres > tamanho_total THEN
        RETURN substring(texto1 FROM 1 FOR tamanho_total - length(texto2) - length(texto3) - 3) || ' ' || texto2 || ' ' || texto3;
    ELSE
        -- Calcula quantos caracteres de preenchimento são necessários
        IF espaco_interno = ' ' THEN
            -- Espaços entre texto1 e texto2
            FOR i IN 1..(tamanho_total - total_caracteres) / 2 LOOP
                espacos_entre1 := espacos_entre1 || ' '; -- Concatena um espaço
            END LOOP;

            -- Espaços entre texto2 e texto3
            FOR i IN 1..(tamanho_total - total_caracteres) - length(espacos_entre1) LOOP
                espacos_entre2 := espacos_entre2 || ' '; -- Concatena um espaço
            END LOOP;
        ELSE
            espacos_entre1 := repeat(espaco_interno, (tamanho_total - total_caracteres) / 2);
            espacos_entre2 := repeat(espaco_interno, (tamanho_total - total_caracteres) - length(espacos_entre1));
        END IF;

        -- Monta o resultado final com os espaços
        RETURN texto1 || espacos_entre1 || texto2 || espacos_entre2 || texto3;
    END IF;
END;
$$ LANGUAGE plpgsql;

-- Função para concatenação dos textos (versão para três expressôes) 
CREATE OR REPLACE FUNCTION completa_textos_trio_edentado(texto1 TEXT, texto2 TEXT, texto3 TEXT)
RETURNS TEXT AS $$
DECLARE
    tamanho_total CONSTANT INT := 46;
    largura_coluna1 CONSTANT INT := 21;
    largura_coluna2 CONSTANT INT := 15;
    largura_coluna3 CONSTANT INT := 10;
    espaco_interno CONSTANT CHAR := ' '; -- Caractere de separação
    total_caracteres INT;
    espacos_entre1 TEXT := ''; -- Espaços entre texto1 e texto2
    espacos_entre2 TEXT := ''; -- Espaços entre texto2 e texto3
    i INT;
BEGIN
      
    -- Remove tag de quebra de linha (Geralmente vem no nome dos meios).
    texto1 := Replace(texto1, '<BR>', ' ');
    
    -- Limita a quantiade de caracteres em cada coluna
    IF (texto2 <> '') AND (texto3 <> '') THEN
       IF length(texto1) >= largura_coluna1 THEN
           texto1 := substring(texto1 from 1 for largura_coluna1);
       ELSE
           texto1 := rpad(texto1, largura_coluna1, ' ');
       END IF;
    END IF;

    -- Limita a quantiade de caracteres em cada coluna
    IF length(texto2) >= largura_coluna2 THEN
        texto2 := substring(texto2 from 1 for largura_coluna2);
--    ELSE
--        texto2 := rpad(texto2, largura_coluna2, ' ');
    END IF;

    -- Limita a quantiade de caracteres em cada coluna
    IF length(texto3) >= largura_coluna3 THEN
        texto3 := substring(texto3 from 1 for largura_coluna3);
    ELSE
        texto3 := lpad(texto3, largura_coluna3, ' ');
    END IF;

    -- Calcula o total de caracteres
    total_caracteres := length(texto1) + length(texto2) + length(texto3); --+ 4; -- +4 para os espaços entre os textos

    -- Se o total de caracteres exceder o tamanho total, corta os textos
    IF total_caracteres > tamanho_total THEN
        RETURN substring(texto1 FROM 1 FOR tamanho_total - length(texto2) - length(texto3) - 3) || ' ' || texto2 || ' ' || texto3;
    ELSE
        -- Calcula quantos caracteres de preenchimento são necessários
        IF espaco_interno = ' ' THEN
            -- Espaços entre texto1 e texto2
            FOR i IN 1..(tamanho_total - total_caracteres) / 2 LOOP
                espacos_entre1 := espacos_entre1 || ' '; -- Concatena um espaço
            END LOOP;

            -- Espaços entre texto2 e texto3
            FOR i IN 1..(tamanho_total - total_caracteres) - length(espacos_entre1) LOOP
                espacos_entre2 := espacos_entre2 || ' '; -- Concatena um espaço
            END LOOP;
        ELSE
            espacos_entre1 := repeat(espaco_interno, (tamanho_total - total_caracteres) / 2);
            espacos_entre2 := repeat(espaco_interno, (tamanho_total - total_caracteres) - length(espacos_entre1));
        END IF;

        -- Monta o resultado final com os espaços
        RETURN texto1 || espacos_entre1 || texto2 || espacos_entre2 || texto3;
    END IF;
END;
$$ LANGUAGE plpgsql;

-- Inicia inserts ou atualiza expressao caso ja exista

-- Cabecalho
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'CABECALHO_CUPOM', TRUE, 
    'SELECT DISTINCT 
       Adicionar_Quebra_Linha() ||
       completa_textos_trio('''', Fil.FilialApelido, '''') ||
       Adicionar_Quebra_Linha() ||
      completa_textos_trio('''', ''CNPJ: '' ||	
      CASE 
          WHEN LENGTH(Par.ParticipanteDocumento) = 11
          THEN Trim(replace(to_char(To_Number(Par.ParticipanteDocumento,''99999999999''), ''000:000:000-00''), '':'', ''.''))
          ELSE Trim(replace(to_char(To_Number(Par.ParticipanteDocumento,''99999999999999''), ''00:000:000/0000-00''), '':'', ''.''))
        END, '''') ||
       Adicionar_Quebra_Linha() ||
       completa_textos_trio('''', ''Bairro: '' || Ende.EnderecoBairro, '''') ||
       Adicionar_Quebra_Linha() ||
       completa_textos_trio('''', ''Complemento: '' || Ende.EnderecoComplemento, '''') ||
       Adicionar_Quebra_Linha() ||
       completa_textos_trio('''',  UF.UnidadeFederativaNome || ''/'' || UF.UnidadeFederativaSigla, '''') ||
       Adicionar_Quebra_Linha() ||
       completa_textos_trio('''', ''Telefone: '' || ''('' || Ende.EnderecoDDD || '')'' || Trim(Ende.EnderecoTelefone), '''') ||
       Adicionar_Quebra_Linha() ||
       ''</linha_simples>'' ||
       Adicionar_Quebra_Linha()	
   AS Texto
 FROM Filial Fil
CROSS JOIN Ambiente Amb
INNER JOIN ECF ECF ON ECF.ECFNomeComputador = Amb.AmbienteNumeroSerie
                  AND (ECF.ECFChave = Amb.AmbienteECFChave OR Amb.AmbienteECFChave = '''')
INNER JOIN Participante Par ON Fil.FilialEnderecoParticipanteId = Par.ParticipanteId
INNER JOIN Endereco Ende ON Fil.FilialEnderecoId = Ende.EnderecoId 
                        AND Fil.FilialEnderecoParticipanteId = Ende.ParticipanteId
INNER JOIN Cidade Cid ON Ende.EnderecoCidadeId = Cid.CidadeId
INNER JOIN UnidadeFederativa UF ON Cid.CidadeUnidadeFederativaId = UF.UnidadeFederativaId
WHERE 1=1
  AND Fil.FilialId = ECF.FilialId
LIMIT 1', 
FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Rodape
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'RODAPE_CUPOM', TRUE, 
    'SELECT 
       Adicionar_Quebra_Linha() ||
       ''</linha_simples>''     || 
       Adicionar_Quebra_Linha(true) ||
       completa_textos_trio(''LOJA: '' || ECF.FilialId::text, ''PDV: '' || ECF.ECFNumero::text, ''OPERADOR: ''  || Usu.UsuarioApelido) ||
       Adicionar_Quebra_Linha() 
    AS Texto
  FROM Transacao Trn
 CROSS JOIN Ambiente Amb
 INNER JOIN ECF ECF ON ECF.ECFNomeComputador = Amb.AmbienteNumeroSerie
                   AND (ECF.ECFChave = Amb.AmbienteECFChave OR Amb.AmbienteECFChave = '''')
 INNER JOIN Filial Fil ON Fil.FilialId = ECF.FilialId
 INNER JOIN Usuario Usu ON Usu.UsuarioId = Trn.TransacaoUsuarioId
 WHERE 1=1
   AND Trn.TransacaoId = $1', 
   FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Reforco
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'REFORCO', TRUE, 
    'SELECT ''{{CABECALHO_CUPOM}}'' ||
        completa_textos(''OPERACAO: '', SR.SangriaReforcoNome) ||
        Adicionar_Quebra_Linha() ||
        completa_textos(''MEIO: '', Meio.MeioDescricao) ||
        Adicionar_Quebra_Linha() ||
        completa_textos(''DOCUMENTO: '', Mov.MovimentoCaixaDocumento) ||
        Adicionar_Quebra_Linha() ||
        completa_textos(''SUPERVISOR: '', Supervisor.UsuarioApelido) ||
        Adicionar_Quebra_Linha() ||
        completa_textos(''OBS: '', 	Mov.MovimentoCaixaHistorico) ||
        Adicionar_Quebra_Linha() ||
        completa_textos(''VALOR: '', Mov.MovimentoCaixaValor::text) ||
       ''{{RODAPE_CUPOM}}''
    AS Texto
  FROM Transacao Trn
 INNER JOIN MovimentoCaixa Mov ON Trn.TransacaoId = Mov.TransacaoId
 INNER JOIN SangriaReforco SR  ON SR.SangriaReforcoId = Mov.SangriaReforcoId
 INNER JOIN Financeiro Fin ON Fin.FinanceiroTransacaoId = Trn.TransacaoId
 INNER JOIN Meio Meio ON Meio.MeioId = Fin.FinanceiroMeioId
 INNER JOIN Caixa Cai ON Cai.CaixaId = Trn.TransacaoCaixaId
 INNER JOIN Usuario Supervisor ON Supervisor.UsuarioId = Cai.CaixaSupervisorId
 WHERE 1=1
   AND Trn.TransacaoId = $1
', FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Retirada
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'RETIRADA', TRUE, 
    'SELECT ''{{CABECALHO_CUPOM}}'' ||
       ''<e><ce>Retirada de Conta Corrente</ce></e>'' ||
       Adicionar_Quebra_Linha(true) ||
       ''</linha_simples>'' ||
       Adicionar_Quebra_Linha(true) ||
       completa_textos(''Cliente: '', Par.ParticipanteDocumento) ||
       Adicionar_Quebra_Linha() ||
       completa_textos('''', Par.ParticipanteRazaoSocial) || 
       Adicionar_Quebra_Linha() ||
       completa_textos(''Tipo: '', 
		CASE 
			WHEN TransacaoTipo = 1 THEN ''Manutencao e suporte''
			WHEN TransacaoTipo = 2 THEN ''Carga''
			WHEN TransacaoTipo = 3 THEN ''Carga inicial''
			WHEN TransacaoTipo = 10 THEN ''Vazia''
			WHEN TransacaoTipo = 50 THEN ''Pre-Venda''
			WHEN TransacaoTipo = 51 THEN ''Cliente via Pre-Venda''
			WHEN TransacaoTipo = 52 THEN ''Solicita credito''
			WHEN TransacaoTipo = 59 THEN ''Cancela Pre-Venda''
			WHEN TransacaoTipo = 60 THEN ''O.S. (Ordem de Servico)''
			WHEN TransacaoTipo = 61 THEN ''Status de O.S.''
			WHEN TransacaoTipo = 62 THEN ''Reimpressao de O.S.''
			WHEN TransacaoTipo = 63 THEN ''Protocolo NFC-e''
			WHEN TransacaoTipo = 64 THEN ''Protocolo Cancelamento''
			WHEN TransacaoTipo = 99 THEN ''Venda''
			WHEN TransacaoTipo = 100 THEN ''Abre Dia''
			WHEN TransacaoTipo = 101 THEN ''Abre Caixa''
			WHEN TransacaoTipo = 102 THEN ''Cupom Fiscal''
			WHEN TransacaoTipo = 103 THEN ''Nota Fiscal''
			WHEN TransacaoTipo = 104 THEN ''Venda (NFCe)''
			WHEN TransacaoTipo = 105 THEN ''Venda (SAT)''
			WHEN TransacaoTipo = 106 THEN ''Recibo de Venda''
			WHEN TransacaoTipo = 107 THEN ''Registro de Nota Manual''
			WHEN TransacaoTipo = 109 THEN ''RPS (Recibo Provisorio de Servicos)''
			WHEN TransacaoTipo = 200 THEN ''Bloqueia o caixa (Shift F10)''
			WHEN TransacaoTipo = 201 THEN ''Propaganda com bloqueio de caixa  (Shift F11)''
			WHEN TransacaoTipo = 202 THEN ''Propaganda sem bloqueio de caixa  (Shift F12)''
			WHEN TransacaoTipo = 203 THEN ''Retirada de cheques''
			WHEN TransacaoTipo = 204 THEN ''Abre gaveta''
			WHEN TransacaoTipo = 290 THEN ''Re-imprime caixa''
			WHEN TransacaoTipo = 299 THEN ''Fecha caixa''
			WHEN TransacaoTipo = 300 THEN ''Estorno de transacoes''
			WHEN TransacaoTipo = 301 THEN ''Devolucao (consulta documento)''
			WHEN TransacaoTipo = 302 THEN ''Devolucao (por TAG ou RFID)''
			WHEN TransacaoTipo = 303 THEN ''Ativacao de Vale Presente (via TEF)''
			WHEN TransacaoTipo = 304 THEN ''Recarga de Vale Presente (via TEF)''
			WHEN TransacaoTipo = 305 THEN ''Consulta de Vale Presente (via TEF)''
			WHEN TransacaoTipo = 306 THEN ''Servicos adicionais (Seguros, garantia estendida ...)''
			WHEN TransacaoTipo = 307 THEN ''Vale gas''
			WHEN TransacaoTipo = 308 THEN ''Vale Presente (proprio)''
			WHEN TransacaoTipo = 309 THEN ''Consulta Vale Presente (proprio)''
			WHEN TransacaoTipo = 310 THEN ''Cupom de Troca''
			WHEN TransacaoTipo = 311 THEN ''Fatura consignados''
			WHEN TransacaoTipo = 312 THEN ''Extrato de consignacao''
			WHEN TransacaoTipo = 313 THEN ''Seleciona tabela de precos''
			WHEN TransacaoTipo = 314 THEN ''Recarga de celular''
			WHEN TransacaoTipo = 380 THEN ''Copia itens de venda''
			WHEN TransacaoTipo = 390 THEN ''Cancelamento de Cupom Fiscal''
			WHEN TransacaoTipo = 399 THEN ''O.S. (Ordem de Servico)''
			WHEN TransacaoTipo = 400 THEN ''Recebimento de debitos de clientes''
			WHEN TransacaoTipo = 401 THEN ''Recebimento de contas externas (Titulos, agua, luz, telefone... )''
			WHEN TransacaoTipo = 402 THEN ''Deposito em conta corrente''
			WHEN TransacaoTipo = 403 THEN ''Retirada de conta corrente''
			WHEN TransacaoTipo = 404 THEN ''Extrato de conta corrente''
			WHEN TransacaoTipo = 405 THEN ''Recebimento de pendencias''
			WHEN TransacaoTipo = 406 THEN ''Extrato de bonus''
			WHEN TransacaoTipo = 407 THEN ''Resgate de bonus''
			WHEN TransacaoTipo = 408 THEN ''Extrato de Debitos''
			WHEN TransacaoTipo = 600 THEN ''LX (Leitura X)''
			WHEN TransacaoTipo = 601 THEN ''LMF (Leitura de Memoria Fiscal)''
			WHEN TransacaoTipo = 602 THEN ''Arq. MF (Gera arquivo de Memoria Fiscal)''
			WHEN TransacaoTipo = 603 THEN ''Arq. MFD (Gera arquivo de Memoria Fiscal Detalhada)''
			WHEN TransacaoTipo = 604 THEN ''Identificacao do PAF-NFC-e''
			WHEN TransacaoTipo = 605 THEN ''Tab. Indice Tecnico Producao''
			WHEN TransacaoTipo = 606 THEN ''Parametros de configuracao''
			WHEN TransacaoTipo = 607 THEN ''Registros do PAF-NFC-e''
			WHEN TransacaoTipo = 608 THEN ''Espelho MFD''
			WHEN TransacaoTipo = 609 THEN ''Envio ao FISCO - REDUCAO Z''
			WHEN TransacaoTipo = 610 THEN ''Envio ao FISCO - ESTOQUE''
			WHEN TransacaoTipo = 611 THEN ''Vendas Identificadas CPF ou CNPJ''
			WHEN TransacaoTipo = 612 THEN ''Requisicoes Externas Registradas''
			WHEN TransacaoTipo = 700 THEN ''Reforco''
			WHEN TransacaoTipo = 799 THEN ''Sangria''
			WHEN TransacaoTipo = 800 THEN ''Cadastro de clientes''
			WHEN TransacaoTipo = 801 THEN ''Impressao de DAV (selecao)''
			WHEN TransacaoTipo = 802 THEN ''Impressao de DAV (abertas)''
			WHEN TransacaoTipo = 803 THEN ''Troca de senha''
			WHEN TransacaoTipo = 804 THEN ''Pendencias de TEF''
			WHEN TransacaoTipo = 805 THEN ''Reimpressao e envio de e-mails (Documentos fiscais da empresa)''
			WHEN TransacaoTipo = 806 THEN ''Reimpressao e envio de e-mails (transacoes deste terminal)''
			WHEN TransacaoTipo = 807 THEN ''Opcoes administrativas de TEF''
			WHEN TransacaoTipo = 808 THEN ''Lancamento de Estoque''
			WHEN TransacaoTipo = 809 THEN ''Reinicia leitora de RFID''
			WHEN TransacaoTipo = 810 THEN ''Mescla de DAVs''
			WHEN TransacaoTipo = 811 THEN ''Lancamentos do caixa''
			WHEN TransacaoTipo = 812 THEN ''Acesso de Supervisor''
			WHEN TransacaoTipo = 813 THEN ''Saida do Modo Supervisor''
			WHEN TransacaoTipo = 850 THEN ''Carga de tabelas do PinPad (Forcada)''
			WHEN TransacaoTipo = 877 THEN ''Ressarcimento de Vale-Compra''
			WHEN TransacaoTipo = 880 THEN ''Alertas do sistema (Shift F4)''
			WHEN TransacaoTipo = 881 THEN ''Status da comunicacao com o Sefaz (Shift F5)''
			WHEN TransacaoTipo = 882 THEN ''Status da comunicacao com o servidor (Shift F6)''
			WHEN TransacaoTipo = 883 THEN ''Status da comunicacao com o TEF (Shift F7)''
			WHEN TransacaoTipo = 890 THEN ''Controle de transmissao''
			WHEN TransacaoTipo = 891 THEN ''Configuracoes de ambiente''
			WHEN TransacaoTipo = 892 THEN ''Configuracoes de formulas''
			WHEN TransacaoTipo = 893 THEN ''Exportacao cadastral (Dados para outros terminais)''
			WHEN TransacaoTipo = 894 THEN ''Sincronismo manual com servidor''
			WHEN TransacaoTipo = 899 THEN ''Relatorio de configuracao''
			WHEN TransacaoTipo = 995 THEN ''Reimpressao em NF-e''
			WHEN TransacaoTipo = 996 THEN ''e-Mail Boleto''
			WHEN TransacaoTipo = 997 THEN ''Envio de e-Mail da NFC-e''
			WHEN TransacaoTipo = 998 THEN ''Carga de tabelas do PinPad''
			WHEN TransacaoTipo = 999 THEN ''Fecha dia''
			END ) || Adicionar_Quebra_Linha() ||
       completa_textos(''Documento: '', Replace(FinCCC.FinanceiroCCCComplemento, ''VC:'', '''')) ||
       Adicionar_Quebra_Linha() ||
       completa_textos_trio('''', ''Valor:'', Trn.TransacaoValor::text) ||
       ''{{RODAPE_CUPOM}}''
   AS Texto
 FROM Transacao Trn
INNER JOIN Financeiro Fin ON Fin.FinanceiroTransacaoId = Trn.TransacaoId
INNER JOIN FinanceiroCCC FinCCC ON FinCCC.FinanceiroId = Fin.FinanceiroId
INNER JOIN Participante Par ON Par.ParticipanteId = Trn.TransacaoParticipanteId
WHERE 1=1
  AND Trn.TransacaoId = $1', 
  FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

 -- ComprovanteContaCorrente
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'COMPROVANTE_CC', TRUE, 
    'SELECT ''{{CABECALHO_CUPOM}}'' ||
       ''<e><ce>Comprovante</ce></e>'' ||
       ''</linha_simples>'' ||
       Adicionar_Quebra_Linha(true) ||
       completa_textos(''Cliente: '', Par.ParticipanteDocumento) ||
       Adicionar_Quebra_Linha() ||
       completa_textos('' '', Par.ParticipanteRazaoSocial) || 
       Adicionar_Quebra_Linha() ||
       completa_textos(''Documento: '', Replace(FinCCC.FinanceiroCCCComplemento, ''VC:'', '''')) ||
       Adicionar_Quebra_Linha() ||
       ''<ae>''  || (select case when Config.ConfiguracaoDevolucaoId = FinCCC.ContaCorrenteId then ''Vale-compra no valor de '' 
			else case when FinCCC.FinanceiroCCCValor < 0 then ''Recebemos o valor de '' 
	                   else ''Autorizo a cobranca de '' 
	              end 
	    end || 
         MoedaSimbolo || '' '' || Abs(FinCCC.FinanceiroCCCValor) || '' (<EXTENSO>), referente <TRANSACAO>'' || -- TODO
        case when Trn.TransacaoTipo in (102,103,104,105,109,301) 
	         then ''.'' 
	         else '' de numerarios em caixa.'' 
	    end || 
		case when Config.ConfiguracaoDevolucaoId = FinCCC.ContaCorrenteId then ''<LF>Valido ate o dia <DATA>.'' else '''' end || 
        case when FinCCC.FinanceiroCCCValor < 0
	         then '''' 
	         else 
                  ''<LF><LF>ASSINATURA: ___________________________'' || 
                  ''<LF><LF>      NOME: ___________________________<LF>'' 
	   end) || ''</ae>'' || 
	Adicionar_Quebra_Linha(true) ||
       completa_textos_trio('''', ''Valor:'', Trn.TransacaoValor::text) ||
       Adicionar_Quebra_Linha() ||
  --     ''<ce>''  || Mov.MovimentoCaixaCodigoBarras || ''</ce>'' ||
       Adicionar_Quebra_Linha(true) ||
  --     ''<ae> <codabar>'' || Mov.MovimentoCaixaCodigoBarras || ''</codabar> </ae>'' ||
       ''{{RODAPE_CUPOM}}''
   AS Texto
 FROM Transacao Trn
--INNER JOIN MovimentoCaixa Mov ON Trn.TransacaoId = Mov.TransacaoId -- TODO: Nesse momento nao existe movimento de caixa?
INNER JOIN Participante Par ON Par.ParticipanteId = Trn.TransacaoParticipanteId
INNER JOIN Financeiro Fin ON Fin.FinanceiroTransacaoId = Trn.TransacaoId
INNER JOIN FinanceiroCCC FinCCC ON FinCCC.FinanceiroId = Fin.FinanceiroId
INNER JOIN ContaCorrente CC ON CC.ContaCorrenteId = FinCCC.FinanceiroCCCId
CROSS JOIN Configuracao Config,
(select MoedaSimbolo, ConfiguracaoDevolucaoId from Moeda, Configuracao where MoedaId = ConfiguracaoMoedaCorrenteId) Moe
WHERE 1=1
  AND Trn.TransacaoId = $1', 
  FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

 -- Movimentacao
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'MOVIMENTACAO', TRUE, 
    'SELECT 
    ''</linha_simples>'' ||
	 Adicionar_Quebra_Linha(true) ||	
    ''<ce>'' || ''Movimentacao'' || ''</ce>'' ||
    Adicionar_Quebra_Linha(true) ||
    ''</linha_simples>'' ||
	 Adicionar_Quebra_Linha(true) ||	
    completa_textos_trio_edentado(''Tipo'', ''Qtde'', ''Valor'') ||
    Adicionar_Quebra_Linha() ||
    completa_textos_trio_edentado(''----'', ''----'', ''-----'') ||
    Adicionar_Quebra_Linha() ||
    COALESCE((
        SELECT 
            string_agg(
                completa_textos_trio_edentado( 
		CASE 
			WHEN TransacaoTipo = 1 THEN ''Manutencao e suporte''
			WHEN TransacaoTipo = 2 THEN ''Carga''
			WHEN TransacaoTipo = 3 THEN ''Carga inicial''
			WHEN TransacaoTipo = 10 THEN ''Vazia''
			WHEN TransacaoTipo = 50 THEN ''Pre-Venda''
			WHEN TransacaoTipo = 51 THEN ''Cliente via Pre-Venda''
			WHEN TransacaoTipo = 52 THEN ''Solicita credito''
			WHEN TransacaoTipo = 59 THEN ''Cancela Pre-Venda''
			WHEN TransacaoTipo = 60 THEN ''O.S. (Ordem de Servico)''
			WHEN TransacaoTipo = 61 THEN ''Status de O.S.''
			WHEN TransacaoTipo = 62 THEN ''Reimpressao de O.S.''
			WHEN TransacaoTipo = 63 THEN ''Protocolo NFC-e''
			WHEN TransacaoTipo = 64 THEN ''Protocolo Cancelamento''
			WHEN TransacaoTipo = 99 THEN ''Venda''
			WHEN TransacaoTipo = 100 THEN ''Abre Dia''
			WHEN TransacaoTipo = 101 THEN ''Abre Caixa''
			WHEN TransacaoTipo = 102 THEN ''Cupom Fiscal''
			WHEN TransacaoTipo = 103 THEN ''Nota Fiscal''
			WHEN TransacaoTipo = 104 THEN ''Venda (NFCe)''
			WHEN TransacaoTipo = 105 THEN ''Venda (SAT)''
			WHEN TransacaoTipo = 106 THEN ''Recibo de Venda''
			WHEN TransacaoTipo = 107 THEN ''Registro de Nota Manual''
			WHEN TransacaoTipo = 109 THEN ''RPS (Recibo Provisorio de Servicos)''
			WHEN TransacaoTipo = 200 THEN ''Bloqueia o caixa (Shift F10)''
			WHEN TransacaoTipo = 201 THEN ''Propaganda com bloqueio de caixa  (Shift F11)''
			WHEN TransacaoTipo = 202 THEN ''Propaganda sem bloqueio de caixa  (Shift F12)''
			WHEN TransacaoTipo = 203 THEN ''Retirada de cheques''
			WHEN TransacaoTipo = 204 THEN ''Abre gaveta''
			WHEN TransacaoTipo = 290 THEN ''Re-imprime caixa''
			WHEN TransacaoTipo = 299 THEN ''Fecha caixa''
			WHEN TransacaoTipo = 300 THEN ''Estorno de transacoes''
			WHEN TransacaoTipo = 301 THEN ''Devolucao''
			WHEN TransacaoTipo = 302 THEN ''Devolucao (por TAG ou RFID)''
			WHEN TransacaoTipo = 303 THEN ''Ativacao de Vale Presente (via TEF)''
			WHEN TransacaoTipo = 304 THEN ''Recarga de Vale Presente (via TEF)''
			WHEN TransacaoTipo = 305 THEN ''Consulta de Vale Presente (via TEF)''
			WHEN TransacaoTipo = 306 THEN ''Servicos adicionais (Seguros, garantia estendida ...)''
			WHEN TransacaoTipo = 307 THEN ''Vale gas''
			WHEN TransacaoTipo = 308 THEN ''Vale Presente (proprio)''
			WHEN TransacaoTipo = 309 THEN ''Consulta Vale Presente (proprio)''
			WHEN TransacaoTipo = 310 THEN ''Cupom de Troca''
			WHEN TransacaoTipo = 311 THEN ''Fatura consignados''
			WHEN TransacaoTipo = 312 THEN ''Extrato de consignacao''
			WHEN TransacaoTipo = 313 THEN ''Seleciona tabela de precos''
			WHEN TransacaoTipo = 314 THEN ''Recarga de celular''
			WHEN TransacaoTipo = 380 THEN ''Copia itens de venda''
			WHEN TransacaoTipo = 390 THEN ''Cancelamento de Cupom Fiscal''
			WHEN TransacaoTipo = 399 THEN ''O.S. (Ordem de Servico)''
			WHEN TransacaoTipo = 400 THEN ''Rec. de Debitos''
			WHEN TransacaoTipo = 401 THEN ''Recebimento de contas externas''
			WHEN TransacaoTipo = 402 THEN ''Deposito em C/C''
			WHEN TransacaoTipo = 403 THEN ''Retirada de C/C''
			WHEN TransacaoTipo = 404 THEN ''Extrato de C/C''
			WHEN TransacaoTipo = 405 THEN ''Recebimento de pendencias''
			WHEN TransacaoTipo = 406 THEN ''Extrato de bonus''
			WHEN TransacaoTipo = 407 THEN ''Resgate de bonus''
			WHEN TransacaoTipo = 408 THEN ''Extrato de Debitos''
			WHEN TransacaoTipo = 600 THEN ''LX (Leitura X)''
			WHEN TransacaoTipo = 601 THEN ''LMF (Leitura de Memoria Fiscal)''
			WHEN TransacaoTipo = 602 THEN ''Arq. MF (Gera arquivo de Memoria Fiscal)''
			WHEN TransacaoTipo = 603 THEN ''Arq. MFD (Gera arquivo de Memoria Fiscal Detalhada)''
			WHEN TransacaoTipo = 604 THEN ''Identificacao do PAF-NFC-e''
			WHEN TransacaoTipo = 605 THEN ''Tab. indice Tecnico Producao''
			WHEN TransacaoTipo = 606 THEN ''Parametros de configuracao''
			WHEN TransacaoTipo = 607 THEN ''Registros do PAF-NFC-e''
			WHEN TransacaoTipo = 608 THEN ''Espelho MFD''
			WHEN TransacaoTipo = 609 THEN ''Envio ao FISCO - REDUcaO Z''
			WHEN TransacaoTipo = 610 THEN ''Envio ao FISCO - ESTOQUE''
			WHEN TransacaoTipo = 611 THEN ''Vendas Identificadas CPF ou CNPJ''
			WHEN TransacaoTipo = 612 THEN ''Requisicoes Externas Registradas''
			WHEN TransacaoTipo = 700 THEN ''Reforco''
			WHEN TransacaoTipo = 799 THEN ''Sangria''
			WHEN TransacaoTipo = 800 THEN ''Cadastro de clientes''
			WHEN TransacaoTipo = 801 THEN ''Impressao de DAV (selecao)''
			WHEN TransacaoTipo = 802 THEN ''Impressao de DAV (abertas)''
			WHEN TransacaoTipo = 803 THEN ''Troca de senha''
			WHEN TransacaoTipo = 804 THEN ''Pendencias de TEF''
			WHEN TransacaoTipo = 805 THEN ''Reimpressao e envio de e-mails''
			WHEN TransacaoTipo = 806 THEN ''Reimpressao e envio de e-mails''
			WHEN TransacaoTipo = 807 THEN ''Opcoes administrativas de TEF''
			WHEN TransacaoTipo = 808 THEN ''Lancamento de Estoque''
			WHEN TransacaoTipo = 809 THEN ''Reinicia leitora de RFID''
			WHEN TransacaoTipo = 810 THEN ''Mescla de DAVs''
			WHEN TransacaoTipo = 811 THEN ''Lancamentos do caixa''
			WHEN TransacaoTipo = 812 THEN ''Acesso de Supervisor''
			WHEN TransacaoTipo = 813 THEN ''Saida do Modo Supervisor''
			WHEN TransacaoTipo = 850 THEN ''Carga de tabelas do PinPad (Forcada)''
			WHEN TransacaoTipo = 877 THEN ''Ressarcimento de Vale-Compra''
			WHEN TransacaoTipo = 880 THEN ''Alertas do sistema (Shift F4)''
			WHEN TransacaoTipo = 881 THEN ''Status da comunicacao com o Sefaz''
			WHEN TransacaoTipo = 882 THEN ''Status da comunicacao com o servidor''
			WHEN TransacaoTipo = 883 THEN ''Status da comunicacao com o TEF''
			WHEN TransacaoTipo = 890 THEN ''Controle de transmissao''
			WHEN TransacaoTipo = 891 THEN ''Configuracoes de ambiente''
			WHEN TransacaoTipo = 892 THEN ''Configuracoes de formulas''
			WHEN TransacaoTipo = 893 THEN ''Exportacao cadastral''
			WHEN TransacaoTipo = 894 THEN ''Sincronismo manual com servidor''
			WHEN TransacaoTipo = 899 THEN ''Relatorio de configuracao''
			WHEN TransacaoTipo = 995 THEN ''Reimpressao em NF-e''
			WHEN TransacaoTipo = 996 THEN ''e-Mail Boleto''
			WHEN TransacaoTipo = 997 THEN ''Envio de e-Mail da NFC-e''
			WHEN TransacaoTipo = 998 THEN ''Carga de tabelas do PinPad''
			WHEN TransacaoTipo = 999 THEN ''Fecha dia''
			END ,
               CAST(Qtd AS VARCHAR), CAST(Valor AS VARCHAR)) || 
               Adicionar_Quebra_Linha() ||
               CASE WHEN TransacaoTipo = 301 THEN completa_textos_trio_edentado(''Vale-Compra Emitido'', CAST(Qtd AS VARCHAR), CAST(Abs(Valor) AS VARCHAR)) || Adicionar_Quebra_Linha()
               ELSE ''''
               END, ''''
            ) 
        FROM (
            SELECT 
                Trn1.TransacaoTipo,
                COUNT(*) AS Qtd,
                CASE 
                      WHEN TransacaoTipo = 301 THEN SUM(Fin.FinanceiroValorPrimeira + (Fin.FinanceiroValorParcelas * Fin.FinanceiroQuantidadeParcelas) - Fin.FinanceiroValorTroco)
                     ELSE SUM(Fin.FinanceiroValorPrimeira + Fin.FinanceiroValorEntrada + (Fin.FinanceiroValorParcelas * Fin.FinanceiroQuantidadeParcelas) - Case When Meio.MeioTipo = ''V'' THEN 0 ELSE Fin.FinanceiroValorTroco END) 
                END AS Valor
            FROM Transacao Trn1
           INNER JOIN Financeiro Fin on Fin.FinanceiroTransacaoId = Trn1.TransacaoId
           INNER JOIN Meio Meio ON Fin.FinanceiroMeioId = Meio.MeioId
           WHERE Trn1.TransacaoCaixaId = Trn.TransacaoCaixaId  --Trn1.TransacaoFim BETWEEN Abertura.TransacaoFim AND now()  -- TODO: Verificar data
             AND Trn1.TransacaoTipo <> 101 -- Abre Caixa 
             AND Trn1.TransacaoTipo <> 299 -- Fecha Caixa 
             AND Trn1.TransacaoSituacao = ''F''
           GROUP BY Trn1.TransacaoTipo
        ) Subquery 
    ), '''') || 
     Adicionar_Quebra_Linha() AS Texto 
FROM Transacao Trn 
INNER JOIN Caixa Cai ON Cai.CaixaId = Trn.TransacaoCaixaId
INNER JOIN Dia Dia ON Dia.DiaId = Cai.CaixaDiaId
INNER JOIN Transacao Abertura ON Abertura.TransacaoId = Dia.DiaAberturaId
--INNER JOIN Transacao Fechamento ON Fechamento.TransacaoId = Dia.DiaFechamentoId
WHERE Trn.TransacaoId = $1', 
FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

 -- Abertura Caixa
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'ABERTURA_CAIXA', TRUE, 
    'SELECT DISTINCT ''{{CABECALHO_CUPOM}}'' ||
       ''<e><ce>ABERTURA DE CAIXA</ce></e>'' ||
       Adicionar_Quebra_Linha(true) ||
       ''</linha_simples>'' ||
       Adicionar_Quebra_Linha(true) ||
       completa_textos(''Usuario'', Usu.UsuarioApelido) ||
       Adicionar_Quebra_Linha() ||
       completa_textos(''Supervisor'', Supervisor.UsuarioApelido) ||
       Adicionar_Quebra_Linha() ||
       completa_textos(''Caixa Numero'', Trn.TransacaoCaixaId::text) ||
       Adicionar_Quebra_Linha() ||
       completa_textos(''Abertura'', Trn.TransacaoInicio::text) ||
       ''{{RODAPE_CUPOM}}''
   AS Texto
 FROM Transacao Trn
CROSS JOIN Ambiente Amb
CROSS JOIN Configuracao Config
INNER JOIN Usuario Usu ON Usu.UsuarioId = Trn.TransacaoUsuarioId
INNER JOIN Caixa Cai ON Cai.CaixaId = Trn.TransacaoCaixaId
INNER JOIN Usuario Supervisor ON Supervisor.UsuarioId = Cai.CaixaSupervisorId
WHERE 1=1
  AND Trn.TransacaoId = $1', 
  FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Sangria
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'SANGRIA', TRUE, 
    'SELECT ''{{CABECALHO_CUPOM}}'' ||
        completa_textos(''OPERACAO: '', SR.SangriaReforcoNome) ||
        Adicionar_Quebra_Linha() ||
        completa_textos(''MEIO: '', Meio.MeioDescricao) ||
        Adicionar_Quebra_Linha() ||
        completa_textos(''DOCUMENTO: '', Mov.MovimentoCaixaDocumento) ||
        Adicionar_Quebra_Linha() ||
        completa_textos(''SUPERVISOR: '', Supervisor.UsuarioApelido) ||
        Adicionar_Quebra_Linha() ||
        completa_textos(''OBS: '', Mov.MovimentoCaixaHistorico) ||
        Adicionar_Quebra_Linha() ||
        completa_textos(''VALOR: '', Mov.MovimentoCaixaValor::text) ||
        Adicionar_Quebra_Linha() ||
        Adicionar_Quebra_Linha() ||
        completa_textos_trio('''', Mov.MovimentoCaixaCodigoBarras, '''') ||
        Adicionar_Quebra_Linha() ||
        completa_textos_trio('''', Mov.MovimentoCaixaCodigoBarrasCodABar, '''') ||
        Adicionar_Quebra_Linha() ||
       ''{{RODAPE_CUPOM}}''
    AS Texto
  FROM Transacao Trn
 INNER JOIN MovimentoCaixa Mov ON Trn.TransacaoId = Mov.TransacaoId
 INNER JOIN SangriaReforco SR  ON SR.SangriaReforcoId = Mov.SangriaReforcoId
 INNER JOIN Financeiro Fin ON Fin.FinanceiroTransacaoId = Trn.TransacaoId
 INNER JOIN Meio Meio ON Meio.MeioId = Fin.FinanceiroMeioId
 INNER JOIN Caixa Cai ON Cai.CaixaId = Trn.TransacaoCaixaId
 INNER JOIN Usuario Supervisor ON Supervisor.UsuarioId = Cai.CaixaSupervisorId
 WHERE 1=1
   AND Trn.TransacaoId = $1
', FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Recibo
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'RECIBO', TRUE, 
    'SELECT ''{{CABECALHO_CUPOM}}'' ||
       ''<e><ce>RECIBO</ce></e>'' ||
       Adicionar_Quebra_Linha(true) ||
       ''</linha_simples>'' ||
       Adicionar_Quebra_Linha(true) ||
       ''<ae>''  || ''CLIENTE: '' || ''</ae>'' || ''<ad>'' || Par.ParticipanteDocumento || ''</ad>'' ||
       Adicionar_Quebra_Linha(true) ||
       ''<ad>''  || Par.ParticipanteRazaoSocial || ''</ad>'' || 
       Adicionar_Quebra_Linha(true) ||
	COALESCE((
		SELECT 
			string_agg(''<ae>'' || Rec.RecebimentoDocumento || ''</ae>'' || ''<ad>'' || Rec.RecebimentoValorRecebido || ''</ad>'' || Adicionar_Quebra_Linha(true), '''') 
		  FROM Recebimento Rec
		 WHERE Rec.TransacaoId = Trn.TransacaoId
		  ), '''') ||    
       ''<ce>''  || ''Total'' || ''</ce>'' || ''<ad>'' || Trn.TransacaoValor || ''</ad>'' ||
       Adicionar_Quebra_Linha(true) ||
       COALESCE((
		SELECT 
			string_agg(''<ce>'' || Meio.MeioDescricao || ''</ce>'' || ''<ad>'' || COALESCE(nullif(FinanceiroValorPrimeira, 0), Fin.FinanceiroValorEntrada) || ''</ad>'' || Adicionar_Quebra_Linha(true), '''') 
		  FROM Financeiro Fin
		 INNER JOIN Meio Meio ON Meio.MeioId = Fin.FinanceiroMeioId
		 WHERE Fin.FinanceiroTransacaoId = Trn.TransacaoId
		), '''')   ||
	Adicionar_Quebra_Linha(true) ||
       ''<ce>''  || ''Valor Recebido'' || ''</ce>'' || ''<ad>'' || 
       COALESCE((
		SELECT 
		   SUM(Rec.RecebimentoValorRecebido)
		  FROM Recebimento Rec
		 WHERE Rec.TransacaoId = Trn.TransacaoId
		), 0)|| ''</ad>'' ||
       ''{{RODAPE_CUPOM}}''
   AS Texto
 FROM Transacao Trn
INNER JOIN Participante Par ON Par.ParticipanteId = Trn.TransacaoParticipanteId
WHERE 1=1
  AND Trn.TransacaoId = $1', 
  FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Deposito
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'DEPOSITO', TRUE, 
    'SELECT ''{{CABECALHO_CUPOM}}'' ||
       ''<e><ce>DEPOSITO</ce></e>'' ||
       Adicionar_Quebra_Linha(true) ||
       ''</linha_simples>'' 	    ||
       Adicionar_Quebra_Linha(true)     ||
       completa_textos(''Cliente: '' , Par.ParticipanteDocumento) ||
       Adicionar_Quebra_Linha() ||
       completa_textos('' '',  Par.ParticipanteRazaoSocial) || 
       Adicionar_Quebra_Linha() ||
       completa_textos(''Tipo: '', 
		CASE 
			WHEN TransacaoTipo = 1 THEN ''Manutencao e suporte''
			WHEN TransacaoTipo = 2 THEN ''Carga''
			WHEN TransacaoTipo = 3 THEN ''Carga inicial''
			WHEN TransacaoTipo = 10 THEN ''Vazia''
			WHEN TransacaoTipo = 50 THEN ''Pre-Venda''
			WHEN TransacaoTipo = 51 THEN ''Cliente via Pre-Venda''
			WHEN TransacaoTipo = 52 THEN ''Solicita credito''
			WHEN TransacaoTipo = 59 THEN ''Cancela Pre-Venda''
			WHEN TransacaoTipo = 60 THEN ''O.S. (Ordem de Servico)''
			WHEN TransacaoTipo = 61 THEN ''Status de O.S.''
			WHEN TransacaoTipo = 62 THEN ''Reimpressao de O.S.''
			WHEN TransacaoTipo = 63 THEN ''Protocolo NFC-e''
			WHEN TransacaoTipo = 64 THEN ''Protocolo Cancelamento''
			WHEN TransacaoTipo = 99 THEN ''Venda''
			WHEN TransacaoTipo = 100 THEN ''Abre Dia''
			WHEN TransacaoTipo = 101 THEN ''Abre Caixa''
			WHEN TransacaoTipo = 102 THEN ''Cupom Fiscal''
			WHEN TransacaoTipo = 103 THEN ''Nota Fiscal''
			WHEN TransacaoTipo = 104 THEN ''Venda (NFCe)''
			WHEN TransacaoTipo = 105 THEN ''Venda (SAT)''
			WHEN TransacaoTipo = 106 THEN ''Recibo de Venda''
			WHEN TransacaoTipo = 107 THEN ''Registro de Nota Manual''
			WHEN TransacaoTipo = 109 THEN ''RPS (Recibo Provisorio de Servicos)''
			WHEN TransacaoTipo = 200 THEN ''Bloqueia o caixa (Shift F10)''
			WHEN TransacaoTipo = 201 THEN ''Propaganda com bloqueio de caixa  (Shift F11)''
			WHEN TransacaoTipo = 202 THEN ''Propaganda sem bloqueio de caixa  (Shift F12)''
			WHEN TransacaoTipo = 203 THEN ''Retirada de cheques''
			WHEN TransacaoTipo = 204 THEN ''Abre gaveta''
			WHEN TransacaoTipo = 290 THEN ''Re-imprime caixa''
			WHEN TransacaoTipo = 299 THEN ''Fecha caixa''
			WHEN TransacaoTipo = 300 THEN ''Estorno de transacoes''
			WHEN TransacaoTipo = 301 THEN ''Devolucao (consulta documento)''
			WHEN TransacaoTipo = 302 THEN ''Devolucao (por TAG ou RFID)''
			WHEN TransacaoTipo = 303 THEN ''Ativacao de Vale Presente (via TEF)''
			WHEN TransacaoTipo = 304 THEN ''Recarga de Vale Presente (via TEF)''
			WHEN TransacaoTipo = 305 THEN ''Consulta de Vale Presente (via TEF)''
			WHEN TransacaoTipo = 306 THEN ''Servicos adicionais (Seguros, garantia estendida ...)''
			WHEN TransacaoTipo = 307 THEN ''Vale gas''
			WHEN TransacaoTipo = 308 THEN ''Vale Presente (proprio)''
			WHEN TransacaoTipo = 309 THEN ''Consulta Vale Presente (proprio)''
			WHEN TransacaoTipo = 310 THEN ''Cupom de Troca''
			WHEN TransacaoTipo = 311 THEN ''Fatura consignados''
			WHEN TransacaoTipo = 312 THEN ''Extrato de consignacao''
			WHEN TransacaoTipo = 313 THEN ''Seleciona tabela de precos''
			WHEN TransacaoTipo = 314 THEN ''Recarga de celular''
			WHEN TransacaoTipo = 380 THEN ''Copia itens de venda''
			WHEN TransacaoTipo = 390 THEN ''Cancelamento de Cupom Fiscal''
			WHEN TransacaoTipo = 399 THEN ''O.S. (Ordem de Servico)''
			WHEN TransacaoTipo = 400 THEN ''Recebimento de debitos de clientes''
			WHEN TransacaoTipo = 401 THEN ''Recebimento de contas externas (Titulos, agua, luz, telefone... )''
			WHEN TransacaoTipo = 402 THEN ''Deposito em conta corrente''
			WHEN TransacaoTipo = 403 THEN ''Retirada de conta corrente''
			WHEN TransacaoTipo = 404 THEN ''Extrato de conta corrente''
			WHEN TransacaoTipo = 405 THEN ''Recebimento de pendencias''
			WHEN TransacaoTipo = 406 THEN ''Extrato de bonus''
			WHEN TransacaoTipo = 407 THEN ''Resgate de bonus''
			WHEN TransacaoTipo = 408 THEN ''Extrato de Debitos''
			WHEN TransacaoTipo = 600 THEN ''LX (Leitura X)''
			WHEN TransacaoTipo = 601 THEN ''LMF (Leitura de Memoria Fiscal)''
			WHEN TransacaoTipo = 602 THEN ''Arq. MF (Gera arquivo de Memoria Fiscal)''
			WHEN TransacaoTipo = 603 THEN ''Arq. MFD (Gera arquivo de Memoria Fiscal Detalhada)''
			WHEN TransacaoTipo = 604 THEN ''Identificacao do PAF-NFC-e''
			WHEN TransacaoTipo = 605 THEN ''Tab. indice Tecnico Producao''
			WHEN TransacaoTipo = 606 THEN ''Parametros de configuracao''
			WHEN TransacaoTipo = 607 THEN ''Registros do PAF-NFC-e''
			WHEN TransacaoTipo = 608 THEN ''Espelho MFD''
			WHEN TransacaoTipo = 609 THEN ''Envio ao FISCO - REDUcaO Z''
			WHEN TransacaoTipo = 610 THEN ''Envio ao FISCO - ESTOQUE''
			WHEN TransacaoTipo = 611 THEN ''Vendas Identificadas CPF ou CNPJ''
			WHEN TransacaoTipo = 612 THEN ''Requisicoes Externas Registradas''
			WHEN TransacaoTipo = 700 THEN ''Reforco''
			WHEN TransacaoTipo = 799 THEN ''Sangria''
			WHEN TransacaoTipo = 800 THEN ''Cadastro de clientes''
			WHEN TransacaoTipo = 801 THEN ''Impressao de DAV (selecao)''
			WHEN TransacaoTipo = 802 THEN ''Impressao de DAV (abertas)''
			WHEN TransacaoTipo = 803 THEN ''Troca de senha''
			WHEN TransacaoTipo = 804 THEN ''Pendencias de TEF''
			WHEN TransacaoTipo = 805 THEN ''Reimpressao e envio de e-mails (Documentos fiscais da empresa)''
			WHEN TransacaoTipo = 806 THEN ''Reimpressao e envio de e-mails (transacoes deste terminal)''
			WHEN TransacaoTipo = 807 THEN ''Opcoes administrativas de TEF''
			WHEN TransacaoTipo = 808 THEN ''Lancamento de Estoque''
			WHEN TransacaoTipo = 809 THEN ''Reinicia leitora de RFID''
			WHEN TransacaoTipo = 810 THEN ''Mescla de DAVs''
			WHEN TransacaoTipo = 811 THEN ''Lancamentos do caixa''
			WHEN TransacaoTipo = 812 THEN ''Acesso de Supervisor''
			WHEN TransacaoTipo = 813 THEN ''Saida do Modo Supervisor''
			WHEN TransacaoTipo = 850 THEN ''Carga de tabelas do PinPad (Forcada)''
			WHEN TransacaoTipo = 877 THEN ''Ressarcimento de Vale-Compra''
			WHEN TransacaoTipo = 880 THEN ''Alertas do sistema (Shift F4)''
			WHEN TransacaoTipo = 881 THEN ''Status da comunicacao com o Sefaz (Shift F5)''
			WHEN TransacaoTipo = 882 THEN ''Status da comunicacao com o servidor (Shift F6)''
			WHEN TransacaoTipo = 883 THEN ''Status da comunicacao com o TEF (Shift F7)''
			WHEN TransacaoTipo = 890 THEN ''Controle de transmissao''
			WHEN TransacaoTipo = 891 THEN ''Configuracoes de ambiente''
			WHEN TransacaoTipo = 892 THEN ''Configuracoes de formulas''
			WHEN TransacaoTipo = 893 THEN ''Exportacao cadastral (Dados para outros terminais)''
			WHEN TransacaoTipo = 894 THEN ''Sincronismo manual com servidor''
			WHEN TransacaoTipo = 899 THEN ''Relatorio de configuracao''
			WHEN TransacaoTipo = 995 THEN ''Reimpressao em NF-e''
			WHEN TransacaoTipo = 996 THEN ''e-Mail Boleto''
			WHEN TransacaoTipo = 997 THEN ''Envio de e-Mail da NFC-e''
			WHEN TransacaoTipo = 998 THEN ''Carga de tabelas do PinPad''
			WHEN TransacaoTipo = 999 THEN ''Fecha dia''
			END ) ||
       Adicionar_Quebra_Linha() ||
       completa_textos(''Documento: '', Replace(FinCCC.FinanceiroCCCComplemento, ''VC:'', '''')) ||
       Adicionar_Quebra_Linha() ||
       completa_textos_trio('''', ''Valor:'', Trn.TransacaoValor::text) ||
       Adicionar_Quebra_Linha() ||
       COALESCE((
		SELECT 
			string_agg(completa_textos_trio('''', Meio.MeioDescricao, COALESCE(nullif(FinanceiroValorPrimeira, 0), Fin.FinanceiroValorEntrada)::text) || Adicionar_Quebra_Linha(), '''') 
		  FROM Financeiro Fin
       INNER JOIN Meio Meio ON Meio.MeioId = Fin.FinanceiroMeioId
       WHERE 1=1
         AND Fin.FinanceiroTransacaoId = Trn.TransacaoId
         AND FinanceiroDebitoCredito = ''C''
		), '''')   ||
       ''{{RODAPE_CUPOM}}''
   AS Texto
 FROM Transacao Trn
INNER JOIN Financeiro Fin ON Fin.FinanceiroTransacaoId = Trn.TransacaoId
INNER JOIN FinanceiroCCC FinCCC ON FinCCC.FinanceiroId = Fin.FinanceiroId
INNER JOIN Participante Par ON Par.ParticipanteId = Trn.TransacaoParticipanteId
WHERE 1=1
  AND Trn.TransacaoId = $1', 
  FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Lancamentos Caixa
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'LANCAMENTOS_CAIXA', TRUE, 
    'SELECT ''{{CABECALHO_CUPOM}}'' ||
       ''<e><ce>LANCAMENTOS DE CAIXA</ce></e>'' ||
       Adicionar_Quebra_Linha() ||
       ''</linha_simples>'' ||
       Adicionar_Quebra_Linha(true) ||
       completa_textos(''Filial / ECF: '', 	Fil.FilialApelido || '' / '' || ECF.ECFNumero) ||  
       Adicionar_Quebra_Linha() ||
       completa_textos(''Caixa Numero: '', Trn.TransacaoCaixaId::text) || 
       Adicionar_Quebra_Linha() ||
       completa_textos(''Abertura: '', Abertura.TransacaoFim::text) || 
       Adicionar_Quebra_Linha() ||
       completa_textos( ''Fechamento: '', CASE WHEN Fechamento.TransacaoFim IS NULL THEN ''AINDA ABERTO'' ELSE CAST(Fechamento.TransacaoFim AS VARCHAR) END) || 
       Adicionar_Quebra_Linha() ||
       completa_textos(''Operador: '', Usu.UsuarioApelido) || 
       Adicionar_Quebra_Linha() ||
       ''{{TOTAL_FUNCOES}}''             ||
       Adicionar_Quebra_Linha() ||
       ''{{CONTAGEM_MEIOS}}''            ||
       Adicionar_Quebra_Linha() ||
       ''{{LANCAMENTOS_FIN}}''         ||
       ''{{RODAPE_CUPOM}}''   
    AS Texto
  FROM Transacao Trn
 CROSS JOIN Ambiente Amb
 CROSS JOIN Configuracao Config
 INNER JOIN Usuario Usu ON Usu.UsuarioId = Trn.TransacaoUsuarioId
 INNER JOIN ECF ECF ON ECF.ECFNomeComputador = Amb.AmbienteNumeroSerie
                   AND (ECF.ECFChave = Amb.AmbienteECFChave OR Amb.AmbienteECFChave = '''')
 INNER JOIN Filial Fil ON Fil.FilialId = ECF.FilialId
 INNER JOIN Caixa Cai ON Cai.CaixaId = Trn.TransacaoCaixaId
 INNER JOIN Dia Dia ON Dia.DiaId = Cai.CaixaDiaId
 INNER JOIN Transacao Abertura ON Abertura.TransacaoId = Dia.DiaAberturaId
  LEFT JOIN Transacao Fechamento ON Fechamento.TransacaoId = Dia.DiaFechamentoId
 INNER JOIN Usuario Supervisor ON Supervisor.UsuarioId = Cai.CaixaSupervisorId
 WHERE 1=1
   AND Trn.TransacaoId = $1
', FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Fechamento Caixa
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'FECHAMENTO_CAIXA', TRUE, 
    'SELECT DISTINCT ''{{CABECALHO_CUPOM}}'' ||
       ''<e><ce>FECHAMENTO DE CAIXA</ce></e>'' ||
       Adicionar_Quebra_Linha(true) ||
       ''</linha_simples>'' ||
       Adicionar_Quebra_Linha(true) ||
       completa_textos(''Filial / ECF: '', Fil.FilialApelido || '' / '' || ECF.ECFNumero) ||
       Adicionar_Quebra_Linha() ||
       completa_textos(''Caixa Numero: '', Trn.TransacaoCaixaId::text)  || 
       Adicionar_Quebra_Linha() ||
       completa_textos(''Operador: '', Usu.UsuarioApelido) || 
       Adicionar_Quebra_Linha() ||
       completa_textos(''Supervisor: '', Supervisor.UsuarioApelido) ||
       Adicionar_Quebra_Linha() ||
       completa_textos(''Abertura: '', to_char(Abertura.TransacaoFim, ''DD-MM-YYYY HH24:MI:SS'')) || -- TODO 
       Adicionar_Quebra_Linha() ||
       completa_textos(''Fechamento: '', to_char(now(), ''DD-MM-YYYY HH24:MI:SS'')) || 
       Adicionar_Quebra_Linha() ||
       ''{{MOVIMENTACAO}}''              ||
       ''{{CONTAGEM_MEIOS}}''            ||
       ''{{SANGRIA_REFORCO}}''           ||
       ''{{CONTAGEM_CEDULAS}}''          ||
       ''{{ATIVIDADES_HORA}}''           ||
       ''{{VALE_COMPRA}}''               ||
       ''{{ITENSVENDAS_ABANDONA}}''      ||
       ''{{RESUMO_CAIXA}}''              ||
       Adicionar_Quebra_Linha() ||
       completa_textos(''Visto do Caixa: '', ''_______________________'') || 
       Adicionar_Quebra_Linha() ||
       completa_textos(''Visto do Gerente: '', ''_______________________'') || 
       ''{{RODAPE_CUPOM}}''      
    AS Texto
  FROM Transacao Trn
 CROSS JOIN Ambiente Amb
 CROSS JOIN Configuracao Config
 INNER JOIN Usuario Usu ON Usu.UsuarioId = Trn.TransacaoUsuarioId
 INNER JOIN ECF ECF ON ECF.ECFNomeComputador = Amb.AmbienteNumeroSerie
                   AND (ECF.ECFChave = Amb.AmbienteECFChave OR Amb.AmbienteECFChave = '''')
 INNER JOIN Filial Fil ON Fil.FilialId = ECF.FilialId
 INNER JOIN Caixa Cai ON Cai.CaixaId = Trn.TransacaoCaixaId
 INNER JOIN Dia Dia ON Dia.DiaId = Cai.CaixaDiaId
 INNER JOIN Transacao Abertura ON Abertura.TransacaoId = Dia.DiaAberturaId
 INNER JOIN Usuario Supervisor ON Supervisor.UsuarioId = Cai.CaixaSupervisorId
 WHERE 1=1
   AND Trn.TransacaoId = $1
', FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Contagem Cedulas
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'CONTAGEM_CEDULAS', TRUE, 
    '  WITH Contagem(x) AS (
SELECT REPLACE(Cai.CaixaCedulas, '' xmlns="TotaliPoS"'', '''')::xml
  FROM Caixa Cai
 INNER JOIN Transacao Trn ON Trn.TransacaoCaixaId = Cai.CaixaId
 WHERE 1=1
   AND Trn.TransacaoId = $1
    
)
SELECT
  ''</linha_simples>'' ||
  Adicionar_Quebra_Linha(true) || 
  ''<ce>'' || ''Contagem de Cedulas'' ||''</ce>'' ||
  Adicionar_Quebra_Linha(true) ||
  ''</linha_simples>'' ||
  Adicionar_Quebra_Linha(true) ||
  completa_textos(''Moeda'', ''Qtde'') || 
  Adicionar_Quebra_Linha() ||
  completa_textos((xpath(''/SDTCedulas/SDTCedulasItem/Valor/text()'', x))[1]::VARCHAR, COALESCE((xpath(''/SDTCedulas/SDTCedulasItem/Contagem/text()'', x))[1], ''0'')::VARCHAR) || Adicionar_Quebra_Linha() || 
  completa_textos((xpath(''/SDTCedulas/SDTCedulasItem/Valor/text()'', x))[2]::VARCHAR, COALESCE((xpath(''/SDTCedulas/SDTCedulasItem/Contagem/text()'', x))[2], ''0'')::VARCHAR) || Adicionar_Quebra_Linha() || 
  completa_textos((xpath(''/SDTCedulas/SDTCedulasItem/Valor/text()'', x))[3]::VARCHAR, COALESCE((xpath(''/SDTCedulas/SDTCedulasItem/Contagem/text()'', x))[3], ''0'')::VARCHAR) || Adicionar_Quebra_Linha() ||
  completa_textos((xpath(''/SDTCedulas/SDTCedulasItem/Valor/text()'', x))[4]::VARCHAR, COALESCE((xpath(''/SDTCedulas/SDTCedulasItem/Contagem/text()'', x))[4], ''0'')::VARCHAR) || Adicionar_Quebra_Linha() ||
  completa_textos((xpath(''/SDTCedulas/SDTCedulasItem/Valor/text()'', x))[5]::VARCHAR, COALESCE((xpath(''/SDTCedulas/SDTCedulasItem/Contagem/text()'', x))[5], ''0'')::VARCHAR) || Adicionar_Quebra_Linha() ||
  completa_textos((xpath(''/SDTCedulas/SDTCedulasItem/Valor/text()'', x))[6]::VARCHAR, COALESCE((xpath(''/SDTCedulas/SDTCedulasItem/Contagem/text()'', x))[6], ''0'')::VARCHAR) || Adicionar_Quebra_Linha() ||
  completa_textos((xpath(''/SDTCedulas/SDTCedulasItem/Valor/text()'', x))[7]::VARCHAR, COALESCE((xpath(''/SDTCedulas/SDTCedulasItem/Contagem/text()'', x))[7], ''0'')::VARCHAR) || Adicionar_Quebra_Linha() ||
  completa_textos((xpath(''/SDTCedulas/SDTCedulasItem/Valor/text()'', x))[8]::VARCHAR, COALESCE((xpath(''/SDTCedulas/SDTCedulasItem/Contagem/text()'', x))[8], ''0'')::VARCHAR) || Adicionar_Quebra_Linha() ||
  completa_textos((xpath(''/SDTCedulas/SDTCedulasItem/Valor/text()'', x))[9]::VARCHAR, COALESCE((xpath(''/SDTCedulas/SDTCedulasItem/Contagem/text()'', x))[9], ''0'')::VARCHAR) || Adicionar_Quebra_Linha() ||
  completa_textos((xpath(''/SDTCedulas/SDTCedulasItem/Valor/text()'', x))[10]::VARCHAR, COALESCE((xpath(''/SDTCedulas/SDTCedulasItem/Contagem/text()'', x))[10], ''0'')::VARCHAR) || Adicionar_Quebra_Linha() ||
  completa_textos((xpath(''/SDTCedulas/SDTCedulasItem/Valor/text()'', x))[11]::VARCHAR, COALESCE((xpath(''/SDTCedulas/SDTCedulasItem/Contagem/text()'', x))[11], ''0'')::VARCHAR) || Adicionar_Quebra_Linha() ||
  completa_textos((xpath(''/SDTCedulas/SDTCedulasItem/Valor/text()'', x))[12]::VARCHAR, COALESCE((xpath(''/SDTCedulas/SDTCedulasItem/Contagem/text()'', x))[12], ''0'')::VARCHAR) || Adicionar_Quebra_Linha() 
  AS Texto
FROM 
 Contagem;', 
 FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Vale Compra
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'VALE_COMPRA', TRUE, 
    'SELECT 
    ''</linha_simples>'' ||
	 Adicionar_Quebra_Linha(true) ||	
    ''<ce>''  || ''Vale-Compra'' || ''</ce>'' ||
    Adicionar_Quebra_Linha(true) ||
    ''</linha_simples>'' ||
    Adicionar_Quebra_Linha(true) ||
       completa_textos(''Total Vale-Compra Recebido'', 
        COALESCE(
       ( 
	  SELECT CAST(SUM(CaiCon.CaixaContagemValorApurado - CaiCon.CaixaContagemValorCancelado) AS VARCHAR)
	    FROM CaixaContagem CaiCon
	   INNER JOIN Meio Mei ON Mei.MeioId = CaiCon.MeioId
	   WHERE 1=1
	     AND CaiCon.CaixaId = Trn.TransacaoCaixaId  --Trn1.TransacaoFim BETWEEN Abertura.TransacaoFim AND now() --Fechamento.TransacaoFim
	     AND Mei.MeioTipo = ''V'' -- Vale Compra
	), ''0'')) ||  
       Adicionar_Quebra_Linha() ||
       completa_textos(''Troco Vale-Compra Recebido'', 
       COALESCE (
	(
	  SELECT CAST(Abs(SUM(FinCCC.FinanceiroCCCValor)) AS VARCHAR)
	    FROM Transacao Trn1
	   INNER JOIN Financeiro Fin ON Fin.FinanceiroTransacaoId = Trn1.TransacaoId
	   INNER JOIN FinanceiroCCC FinCCC ON FinCCC.FinanceiroId = Fin.FinanceiroId
	   INNER JOIN Meio Mei ON Mei.MeioId = Fin.FinanceiroMeioId
	   WHERE 1=1
             AND Trn1.TransacaoCaixaId = Trn.TransacaoCaixaId  --Trn1.TransacaoFim BETWEEN Abertura.TransacaoFim AND now() --Fechamento.TransacaoFim
	     AND Fin.FinanceiroValorPrimeira < 0 --AND Fin.FinanceiroValorEntrada < 0
	     AND Mei.MeioTipo = ''V'' -- Vale Compra
	     AND FinCCC.FinanceiroCCCTipo = ''T'' -- Troco
	), ''0'')) ||  
	Adicionar_Quebra_Linha() ||
       completa_textos(''Total Vale-Compra Utilizado'', 
       CAST (
	COALESCE(
       ( 
	  SELECT SUM(CaiCon.CaixaContagemValorApurado - CaiCon.CaixaContagemValorCancelado) 
	    FROM CaixaContagem CaiCon
	   INNER JOIN Meio Mei ON Mei.MeioId = CaiCon.MeioId
	   WHERE 1=1
	     AND CaiCon.CaixaId = Trn.TransacaoCaixaId  --Trn1.TransacaoFim BETWEEN Abertura.TransacaoFim AND now() --Fechamento.TransacaoFim
	     AND Mei.MeioTipo = ''V'' -- Vale Compra
	), 0) - 
	COALESCE (
	(
	  SELECT Abs(SUM(FinCCC.FinanceiroCCCValor))
	    FROM Transacao Trn1
	   INNER JOIN Financeiro Fin ON Fin.FinanceiroTransacaoId = Trn1.TransacaoId
	   INNER JOIN FinanceiroCCC FinCCC ON FinCCC.FinanceiroId = Fin.FinanceiroId
	   INNER JOIN Meio Mei ON Mei.MeioId = Fin.FinanceiroMeioId
	   WHERE 1=1
             AND Trn1.TransacaoCaixaId = Trn.TransacaoCaixaId  --Trn1.TransacaoFim BETWEEN Abertura.TransacaoFim AND now() --Fechamento.TransacaoFim
	     AND Fin.FinanceiroValorPrimeira < 0 --AND Fin.FinanceiroValorEntrada < 0
	     AND Mei.MeioTipo = ''V'' -- Vale Compra
	     AND FinCCC.FinanceiroCCCTipo = ''T'' -- Troco
	), 0) AS VARCHAR)) ||
	Adicionar_Quebra_Linha() 
    AS Texto
  FROM Transacao Trn
 INNER JOIN Caixa Cai ON Cai.CaixaId = Trn.TransacaoCaixaId
 INNER JOIN Dia Dia ON Dia.DiaId = Cai.CaixaDiaId
 INNER JOIN Transacao Abertura ON Abertura.TransacaoId = Dia.DiaAberturaId
-- INNER JOIN Transacao Fechamento ON Fechamento.TransacaoId = Dia.DiaFechamentoId
 WHERE 1=1
   AND Trn.TransacaoId = $1', 
   FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Sangria e Reforco
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'SANGRIA_REFORCO', TRUE, 
    'SELECT 
     ''</linha_simples>'' ||
	  Adicionar_Quebra_Linha(true) ||	
     ''<ce>''  || ''Sangrias e Reforcos'' || ''</ce>'' ||
     Adicionar_Quebra_Linha(true) ||
     ''</linha_simples>'' ||
	  Adicionar_Quebra_Linha(true) ||	
       completa_textos(''SANGRIA'', 
       COALESCE(
       (
	  SELECT CAST(SUM(Trn1.TransacaoValor) AS VARCHAR)
            FROM Transacao Trn1
           WHERE 1=1
             AND Trn1.TransacaoCaixaId = Trn.TransacaoCaixaId  --Trn1.TransacaoFim BETWEEN Abertura.TransacaoFim AND now() -- TODO: Verificar data
	     AND Trn1.TransacaoTipo = 799 -- Sangria
           GROUP BY Trn1.TransacaoTipo   
       ), ''0'')) ||  
       Adicionar_Quebra_Linha() ||
       completa_textos(''REFORCO'', 
       COALESCE(
       (
	  SELECT CAST(SUM(Trn1.TransacaoValor) AS VARCHAR)
            FROM Transacao Trn1
           WHERE 1=1
             AND Trn1.TransacaoCaixaId = Trn.TransacaoCaixaId  --Trn1.TransacaoFim BETWEEN Abertura.TransacaoFim AND now() -- TODO: Verificar data
	     AND Trn1.TransacaoTipo = 700 -- Reforco
           GROUP BY Trn1.TransacaoTipo   
       ), ''0'')) ||
       Adicionar_Quebra_Linha()
       AS Texto
 FROM Transacao Trn 
INNER JOIN Caixa Cai ON Cai.CaixaId = Trn.TransacaoCaixaId
INNER JOIN Dia Dia ON Dia.DiaId = Cai.CaixaDiaId
INNER JOIN Transacao Abertura ON Abertura.TransacaoId = Dia.DiaAberturaId
--INNER JOIN Transacao Fechamento ON Fechamento.TransacaoId = Dia.DiaFechamentoId
WHERE Trn.TransacaoId = $1', 
FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Contagem Meios de Pagamento
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'CONTAGEM_MEIOS', TRUE, 
    'SELECT 
    ''</linha_simples>'' ||
	 Adicionar_Quebra_Linha(true) ||	
    ''<ce>''  || ''CONTAGEM POR MEIO DE PAGAMENTO'' || ''</ce>'' ||
    Adicionar_Quebra_Linha(true) ||
    ''</linha_simples>'' ||
	 Adicionar_Quebra_Linha(true) ||	
      completa_textos_trio_edentado(''Meio'', ''Total'', ''Diferenca'') ||  
       Adicionar_Quebra_Linha() ||
       completa_textos_trio_edentado(''----'', ''-----'', ''---------'') ||   
       Adicionar_Quebra_Linha() || 
       COALESCE((
          SELECT 
              string_agg(
                  completa_textos_trio_edentado(MEIO, CAST(Valor AS VARCHAR), CAST(Diferenca AS VARCHAR)) || Adicionar_Quebra_Linha(), '''')
          FROM (
              SELECT 
                  Meio.MeioDescricao AS MEIO,
                  COALESCE(SUM(
                      CASE 
                          WHEN Meio.MeioTipo = ''V'' AND Fin.FinanceiroDebitoCredito = ''C'' THEN ABS(Fin.FinanceiroValorPrimeira)
                          ELSE Fin.FinanceiroValorPrimeira
                      END
                      + Fin.FinanceiroValorEntrada 
                      + (Fin.FinanceiroValorParcelas * Fin.FinanceiroQuantidadeParcelas) 
                      - CASE 
                          WHEN Meio.MeioTipo = ''V'' AND Fin.FinanceiroDebitoCredito = ''D'' THEN 0 
                          ELSE Fin.FinanceiroValorTroco 
                        END
                  ), 0) AS Valor,
                  COALESCE(CaiCon.Diferenca, 0) AS Diferenca
              FROM Transacao Trn1
              INNER JOIN Financeiro Fin ON Fin.FinanceiroTransacaoId = Trn1.TransacaoId
              INNER JOIN Meio Meio ON Fin.FinanceiroMeioId = Meio.MeioId
              LEFT JOIN (
                  SELECT CaixaId, MeioId,
                         SUM(CaixaContagemValorApurado - CaixaContagemValorCancelado - CaixaContagemValorContado) AS Diferenca
                  FROM CaixaContagem
                  GROUP BY CaixaId, MeioId
              ) CaiCon ON CaiCon.MeioId = Meio.MeioId
                     AND CaiCon.CaixaId = Trn1.TransacaoCaixaId 
              WHERE Trn1.TransacaoCaixaId = Trn.TransacaoCaixaId
                AND Fin.FinanceiroSituacao <> ''C''
                AND Meio.MeioTipo <> ''N''
                AND (Meio.MeioTipo <> ''V'' OR Fin.FinanceiroDebitoCredito = ''C'')
              GROUP BY Meio.MeioDescricao, CaiCon.Diferenca
          ) Subquery
      ), '''') AS Texto
FROM Transacao Trn 
INNER JOIN Caixa Cai ON Cai.CaixaId = Trn.TransacaoCaixaId
INNER JOIN Dia Dia ON Dia.DiaId = Cai.CaixaDiaId
INNER JOIN Transacao Abertura ON Abertura.TransacaoId = Dia.DiaAberturaId
--INNER JOIN Transacao Fechamento ON Fechamento.TransacaoId = Dia.DiaFechamentoId
WHERE Trn.TransacaoId = $1', 
FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Resumo
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'RESUMO_CAIXA', TRUE, 
    'SELECT 
    ''</linha_simples>'' ||
	 Adicionar_Quebra_Linha(true) ||	
    ''<ce>''  || ''Resumo'' || ''</ce>'' ||
    Adicionar_Quebra_Linha(true) ||
    ''</linha_simples>'' ||
    Adicionar_Quebra_Linha(true) ||
       completa_textos(''Qtd. de Operacoes'', 
       COALESCE( -- Quantidade de operacoes dentro do periodo em que o caixa ficou aberto sem contar as aberturas de caixa
       (
	  SELECT CAST(COUNT(*) AS VARCHAR)
            FROM Transacao Trn1
           WHERE 1=1
             AND Trn1.TransacaoCaixaId = Trn.TransacaoCaixaId  --Trn1.TransacaoFim BETWEEN Abertura.TransacaoFim AND now() -- TODO: Verificar data
             AND Trn1.TransacaoTipo <> 101 -- Abre Caixa 
             AND Trn1.TransacaoTipo <> 299 -- Fecha Caixa 
       ), ''0'')) ||  
       Adicionar_Quebra_Linha() ||
       completa_textos(''Total Movimentado'', 
       COALESCE( -- Valor total das contagens de caixa durante o periodo que o caixa ficou aberto
       (
	  SELECT CAST(SUM(CaiCon.CaixaContagemValorApurado - CaiCon.CaixaContagemValorCancelado) AS VARCHAR)
            FROM CaixaContagem CaiCon
            INNER JOIN Meio Mei  ON Mei.MeioId = CaiCon.MeioId 
            WHERE 1=1
             AND CaiCon.CaixaId = Trn.TransacaoCaixaId  --Trn1.TransacaoFim BETWEEN Abertura.TransacaoFim AND now() -- TODO: Verificar data
             --AND Fin.FinanceiroDebitoCredito = ''C''
             AND Mei.MeioTipo <> ''N''
       ), ''0'')) ||  
       Adicionar_Quebra_Linha() ||
       completa_textos(''Total Vendas (Excluindo Estornos)'', 
       COALESCE( -- Somatorio do valor das transacoes substraido pelo valor dos cancelamentos
       (
	  COALESCE ( (
	  SELECT SUM(Trn1.TransacaoValor) - SUM(COALESCE(Fin.FinanceiroValorTroco, 0))
            FROM Transacao Trn1
            LEFT JOIN Financeiro Fin ON Fin.financeirotransacaoid = Trn1.TransacaoId
                                    AND Fin.FinanceiroValorTroco > 0
           WHERE 1=1
             AND Trn1.TransacaoCaixaId = Trn.TransacaoCaixaId  
	          AND Trn1.TransacaoTipo in (99, 102, 104, 109, 105, 103, 107, 106) 
             AND Trn1.TransacaoSituacao = ''F''
	  ), 0)-- TransacaoEhVenda
	  -
	  COALESCE( (
	  SELECT SUM(Can.CancelamentoValor)
            FROM Transacao Trn1
           INNER JOIN Cancelamento Can ON Can.CancelamentoTransacaoId = Trn1.TransacaoId
           WHERE 1=1
             AND Trn1.TransacaoCaixaId = Trn.TransacaoCaixaId  --Trn1.TransacaoFim BETWEEN Abertura.TransacaoFim AND now() -- TODO: Verificar data
	     AND Trn1.TransacaoTipo in (99, 102, 104, 109, 105, 103, 107, 106) 
	    ), 0)-- TransacaoEhVenda
       )::text, ''0'')) ||  
       Adicionar_Quebra_Linha() ||
       CASE 
        WHEN Cai.CaixaSemContagem = True 
	THEN Adicionar_Quebra_Linha() || Adicionar_Quebra_Linha() || ''<ce>''  || ''*** SEM CONTAGEM DE VALORES. ***'' || ''</ce>''
	ELSE completa_textos(''Diferenca Apurada'', 
	  COALESCE(
	       ( 
		  SELECT CAST(SUM(CaiCon.CaixaContagemValorApurado /*+ CaiCon.CaixaContagemValorPendente*/ - CaiCon.CaixaContagemValorCancelado - CaiCon.CaixaContagemValorContado) AS VARCHAR) -- TODO Pendencia
		    FROM Transacao Trn1
		   INNER JOIN Financeiro Fin ON Fin.FinanceiroTransacaoId = Trn1.TransacaoId
		   INNER JOIN Meio Mei ON Mei.MeioId = Fin.FinanceiroMeioId
		   INNER JOIN CaixaContagem CaiCon ON CaiCon.MeioId = Mei.MeioId
						  AND CaiCon.CaixaId = Trn1.TransacaoCaixaId
		   WHERE 1=1
		     AND Trn1.TransacaoCaixaId = Trn.TransacaoCaixaId  --Trn1.TransacaoFim BETWEEN Abertura.TransacaoFim AND now() 
           AND Fin.FinanceiroDebitoCredito = ''C''
		), ''0'')) ||
		Adicionar_Quebra_Linha() ||
	     completa_textos(''Diferenca Real'', ''_______________________'')
       END ||
       Adicionar_Quebra_Linha(true)        
       AS Texto
 FROM Transacao Trn 
INNER JOIN Caixa Cai ON Cai.CaixaId = Trn.TransacaoCaixaId
INNER JOIN Dia Dia ON Dia.DiaId = Cai.CaixaDiaId
INNER JOIN Transacao Abertura ON Abertura.TransacaoId = Dia.DiaAberturaId
--INNER JOIN Transacao Fechamento ON Fechamento.TransacaoId = Dia.DiaFechamentoId
WHERE Trn.TransacaoId = $1', 
FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'ITENSVENDAS_ABANDONA', TRUE, 
    'SELECT 
    ''</linha_simples>'' ||
	 Adicionar_Quebra_Linha(true) ||	
    ''<ce>''  || ''Vendas/Itens Cancelados'' || ''</ce>'' ||
    Adicionar_Quebra_Linha(true) ||
    ''</linha_simples>'' ||
    Adicionar_Quebra_Linha(true) ||
       completa_textos(''Quantidade Total de Vendas Canceladas'', 
        COALESCE(
       ( 
	  SELECT CAST(COUNT(*) AS VARCHAR)
	    FROM Documento Doc
	   INNER JOIN Transacao Trn1 on Trn1.TransacaoId = Doc.DocumentoTransacaoId
	   WHERE 1=1
	     AND Trn1.TransacaoCaixaId = Trn.TransacaoCaixaId 
	     AND Trn1.TransacaoTipoVenda in (1, 2, 3, 5)
	     AND Doc.DocumentoMotivoVendasPerdidasId is not null
	), ''0'')) ||
       Adicionar_Quebra_Linha() ||
       completa_textos(''Quantidade Total de Itens Cancelados'', 
        COALESCE(
       ( 
	  SELECT CAST(Trunc(Abs(SUM(DocumentoItemQuantidade))) AS VARCHAR)
	    FROM DocumentoItem DocIte
	   INNER JOIN Documento Doc on Doc.DocumentoId = DocIte.DocumentoId
	   INNER JOIN Transacao Trn1 on Trn1.TransacaoId = Doc.DocumentoTransacaoId
	   WHERE 1=1
	     AND Trn1.TransacaoCaixaId = Trn.TransacaoCaixaId 
	     AND Trn1.TransacaoTipoVenda in (1, 2, 3, 5)
	     AND DocIte.DocumentoItemMotivoVendasPerdidasId is not null
	), ''0'')) ||  
       Adicionar_Quebra_Linha() ||
       completa_textos(''Valor Total dos Cancelamentos'', 
       COALESCE (
	(
	  SELECT CAST(Round(Abs(SUM(DocIte.DocumentoItemValorUnitario * DocIte.DocumentoItemQuantidade)), 2) AS VARCHAR)
	    FROM DocumentoItem DocIte
	   INNER JOIN Documento Doc on Doc.DocumentoId = DocIte.DocumentoId
	   INNER JOIN Transacao Trn1 on Trn1.TransacaoId = Doc.DocumentoTransacaoId
	   WHERE 1=1
	     AND Trn1.TransacaoCaixaId = Trn.TransacaoCaixaId 
	     AND Trn1.TransacaoTipoVenda in (1, 2, 3, 5)
	     AND DocIte.DocumentoItemMotivoVendasPerdidasId is not null
	), ''0'')) ||
	Adicionar_Quebra_Linha() 
    AS Texto
  FROM Transacao Trn
 INNER JOIN Caixa Cai ON Cai.CaixaId = Trn.TransacaoCaixaId
 INNER JOIN Dia Dia ON Dia.DiaId = Cai.CaixaDiaId
 INNER JOIN Transacao Abertura ON Abertura.TransacaoId = Dia.DiaAberturaId
 WHERE 1=1
   AND Trn.TransacaoId = $1', 
   FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Termo de Confissao de Divida
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'CONFISSAO_DIVIDA', TRUE, 
    'SELECT 
    ''<CABECALHO>'' ||
    Adicionar_Quebra_Linha() ||
    ''TERMO DE CONFISSÃO DE DÍVIDA'' ||
    Adicionar_Quebra_Linha()  ||
    ''Eu, '' || Par.ParticipanteLista || '', portador(a) do CPF '' ||
    CASE WHEN Par.ParticipanteDocumento <> '''' 
         THEN replace(to_char(To_Number(Par.ParticipanteDocumento,''99999999999''), ''000:000:000-00''), '':'', ''.'') 
    END ||
    '' domiciliado(a) no endereço: '' ||
    COALESCE(Ede.EnderecoEndereco, EdePric.EnderecoEndereco) || '', '' || 
    COALESCE(Ede.EnderecoNumero, EdePric.EnderecoNumero) ||
    CASE WHEN COALESCE(Ede.EnderecoComplemento, ''X'') = ''X''
        THEN '' '' 
        ELSE '' - '' || COALESCE(Ede.EnderecoComplemento, EdePric.EnderecoComplemento)
    END || 
    '' - BAIRRO '' || COALESCE(Ede.EnderecoBairro, EdePric.EnderecoBairro) || '' - '' ||
    COALESCE(Cid.CidadeNome, '''') || ''-'' || COALESCE(UF.UnidadeFederativaSigla, '''') ||'','' ||
    ''abaixo assinado, declaro ser devedor(a) '' ||
    ''da '' || Fil.FilialApelido || '' através de contrato '' ||
    ''de Nº '' || Par.ParticipanteChave ||
    '' e, após conferidos detalhadamente os cálculos de meu '' ||
    ''débito, reconheço a exatidão dos mesmos, que perfazem '' ||
    ''até a presente data o valor de '' ||
    ''<VALOR> (<EXTENSO>) '' ||
    ''referente ao Documento Fiscal NFe/NFCe Nº <NOTA> de '' || to_char(Trn.TransacaoInicio, ''DD/MM/YYYY às HH24:MI:SS'' ) ||
    '' obrigando-me a efetuar o pagamento das parcelas '' || 
    ''negociadas e aqui contratadas, que são as seguintes:'' ||
    Adicionar_Quebra_Linha() ||
    Adicionar_Quebra_Linha() ||
    ''<PARCELAS>'' ||
    Adicionar_Quebra_Linha()  ||
    ''Obrigando-me a efetuar o pagamento das parcelas '' ||
    ''até o dia do vencimento, em estabelecimentos da '' || 
    ''Credora, ou onde ela indicar. A falta de '' ||
    ''pagamento de quaisquer das parcelas no '' ||
    ''respectivo vencimento produzirá o vencimento '' ||
    ''imediato e antecipado do saldo, dando à Credora '' ||
    ''pleno direito de proceder à Cobrança Judicial.'' ||
    Adicionar_Quebra_Linha()  ||
    COALESCE(Cid.CidadeNome, '''') || ''-'' || COALESCE(UF.UnidadeFederativaSigla, '''') ||
    Adicionar_Quebra_Linha()  ||
    Adicionar_Quebra_Linha()  ||
    ''______________________________________'' ||
    Adicionar_Quebra_Linha()  ||
    ''Assinatura do Cliente'' ||
    Adicionar_Quebra_Linha()  ||
    ''<RODAPE>''
    AS Texto
 FROM Transacao Trn 
CROSS JOIN Ambiente Amb
INNER JOIN ECF ECF ON ECF.ECFNomeComputador = Amb.AmbienteNumeroSerie
                  AND (ECF.ECFChave = Amb.AmbienteECFChave OR Amb.AmbienteECFChave = '''')
INNER JOIN Filial Fil ON Fil.FilialId = ECF.filialId 
INNER JOIN Documento Doc ON Doc.DocumentoTransacaoId = Trn.TransacaoId
INNER JOIN Participante Par ON Par.ParticipanteId = Trn.TransacaoParticipanteId
INNER JOIN Endereco Ede ON Ede.ParticipanteId = Doc.DocumentoEnderecoFaturaParticipanteId 
                       AND Ede.EnderecoId = Doc.DocumentoEnderecoFaturaId
 LEFT JOIN Endereco EdePric ON EdePric.EnderecoChave = Par.ParticipanteEnderecoPrincipalChave
    	                  AND EdePric.ParticipanteId = Par.ParticipanteId
INNER JOIN Cidade Cid ON Cid.CidadeId = COALESCE(Ede.EnderecoCidadeId, EdePric.EnderecoCidadeId)
INNER JOIN UnidadeFederativa UF ON UF.UnidadeFederativaId = Cid.CidadeUnidadeFederativaId
 LEFT JOIN Categoria1 Ctg1 ON Ctg1.Categoria1Id = Par.ParticipanteCategoria1Id
WHERE 1=1
  AND Trn.TransacaoId = $1 
  AND Par.ParticipanteTipoPessoa = 0 
  AND Ctg1.Categoria1Nome <> ''FUNCIONARIO'' ', 
FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

/*
-- Nota de Promissoria
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'NOTA_PROMISSORIA', TRUE, 
    ' select
		''<CABECALHO>'' ||
		Adicionar_Quebra_Linha()  											||
		completa_textos_trio(''PARCELA'', ''DATA VENCIMENTO'', ''VALOR'')	||
		Adicionar_Quebra_Linha() 											||
		completa_textos_trio(''<NUMERO>'', ''<DATA>'', ''<VALOR>'') 		||
		Adicionar_Quebra_Linha()  											||
		Adicionar_Quebra_Linha()  											||
		Adicionar_Quebra_Linha()  											||
		completa_textos_trio('''', ''_________________________________________'', '''')	||
		Adicionar_Quebra_Linha()  											||
		completa_textos_trio('''', ''Assinatura do Cliente'', '''')			||
		Adicionar_Quebra_Linha() 											||
		''<RODAPE>''
		as Texto
		from Transacao Trn 
		left join Cupom Cup on Cup.CupomId = Trn.TransacaoCupomId
		left join Documento Doc on Doc.DocumentoTransacaoId = Trn.TransacaoId
		left join Participante Par on Par.ParticipanteId = Trn.TransacaoParticipanteId
		left join Categoria1 Ct1 on Ct1.Categoria1Id = Par.ParticipanteCategoria1Id
		where Trn.TransacaoId = $1
		and Par.ParticipanteTipoPessoa = 0
		and Ct1.Categoria1Nome <> ''FUNCIONARIO'' ', 
FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();
*/

-- Nota de Promissoria
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'NOTA_PROMISSORIA', TRUE, 
    '', 
FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Cancelamento/Estorno de Transacao
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'CANCELAMENTO_ESTORNO', TRUE, 
    'SELECT ''{{CABECALHO_CUPOM}}'' ||
''<e><ce>CANCELAMENTO/ESTORNO</ce></e>'' ||
Adicionar_Quebra_Linha(true) ||
completa_textos(''CLIENTE: '', COALESCE(Can.CancelamentoParticipanteDocumento, '''')) ||
Adicionar_Quebra_Linha() ||
completa_textos('' '', COALESCE(Can.CancelamentoParticipanteNome, '''')) ||
Adicionar_Quebra_Linha() ||
completa_textos(''CANCELAMENTO DO COO/NOTA: '', COALESCE(Doc.DocumentoNumero, Cup.CupomNumero)::text) ||
Adicionar_Quebra_Linha() ||
completa_textos(''VALOR: '', COALESCE(Can.CancelamentoValor, 0)::text) ||
''{{RODAPE_CUPOM}}''
AS Texto
FROM Transacao Trn
INNER JOIN Cancelamento Can ON Can.CancelamentoTransacao2Id = Trn.TransacaoId
INNER JOIN Transacao TrnEstorno ON TrnEstorno.TransacaoId = Can.CancelamentoTransacaoId
LEFT JOIN Documento Doc ON Doc.DocumentoTransacaoId = TrnEstorno.TransacaoId
LEFT JOIN Cupom Cup ON Cup.CupomId = TrnEstorno.TransacaoCupomId
WHERE 1=1
AND Trn.TransacaoId = $1 ', 
FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Desenvolvido para o cliente Papel&Cia
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'UPDATE_PLANO', TRUE, 
    'UPDATE Plano
        SET PlanoNegociacao = True
      WHERE PlanoId = $1
        AND (PlanoApelido like ''%NEG%'' OR PlanoApelido like ''%\%%''); -- Planos com "NEG" ou "%" na descricao ', 
FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Desenvolvido para cliente Papel&Cia
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'TABELA_PRECO', TRUE, 
    'SELECT Tab.TabelaPrecoId
       FROM Transacao 
       LEFT JOIN Ambiente ON AmbienteId = 1
       LEFT JOIN ECF ECF ON ECFNomeComputador = AmbienteNumeroSerie
       LEFT JOIN Filial Fil ON Fil.FilialId = ECF.FilialId
       LEFT JOIN TabelaPreco Var ON Var.TabelaPrecoId = ECFTabelaPrecoId
       LEFT JOIN Participante ON ParticipanteId = TransacaoParticipanteId
       LEFT JOIN TabelaPreco Tab ON Tab.TabelaPrecoChave = CASE WHEN FilialChave = ''006'' AND ParticipanteTipoPessoa = 1 THEN ''0025'' 
                                                                WHEN FilialChave = ''006'' AND NOT ParticipanteConsumidorFinal THEN ''0014''
                                                           ELSE Var.TabelaPrecoChave END
      WHERE TransacaoId = $1 ', 
FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Extrato de Conta Corrente OU Fidelidade
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'EXTRATO_CC_FID', TRUE, 
    'SELECT DISTINCT ''{{CABECALHO_CUPOM}}'' ||
''<e><ce>EXTRATO - '' || CC.ContaCorrenteNome || ''</ce></e>'' ||
Adicionar_Quebra_Linha(true) ||
completa_textos(''DE: '', TO_CHAR(ECC.ExtratoContaCorrenteDataInicio, ''DD-MM-YYYY'')) ||
Adicionar_Quebra_Linha() ||
completa_textos(''ATE: '', TO_CHAR(ECC.ExtratoContaCorrenteDataFim, ''DD-MM-YYYY'')) ||
Adicionar_Quebra_Linha() ||
''</linha_simples>'' ||
Adicionar_Quebra_Linha(true) ||
completa_textos(''CLIENTE: '', Par.ParticipanteDocumento) ||
Adicionar_Quebra_Linha() ||
''<ad>'' || Par.ParticipanteRazaoSocial || ''</ad>'' ||
Adicionar_Quebra_Linha(true) ||
completa_textos(''DATA-DOCUMENTO-HISTORICO'', ''VALOR'') ||
Adicionar_Quebra_Linha() ||
''</linha_simples>'' ||
Adicionar_Quebra_Linha(true) ||
COALESCE((
	SELECT string_agg(completa_textos(TO_CHAR(ECC1.ExtratoContaCorrenteDataHora, ''DD-MM-YYYY'') || ''-'' || ECC1.ExtratoContaCorrenteDocumento || ''-'' || ECC1.ExtratoContaCorrenteDetalhes, ECC1.ExtratoContaCorrenteValor::text)|| Adicionar_Quebra_Linha(), '''') 
	  FROM ExtratoContaCorrente ECC1
	 WHERE ECC1.ExtratoContaCorrenteTransacaoId = Trn.TransacaoId
  ), '''') ||    
Adicionar_Quebra_Linha() ||
completa_textos_trio('''', ''SALDO:'', COALESCE((SELECT SUM(ECC2.ExtratoContaCorrenteValor) FROM ExtratoContaCorrente ECC2 Where ECC2.ExtratoContaCorrenteTransacaoId = Trn.TransacaoId), 0)::text) ||
''{{RODAPE_CUPOM}}''
AS Texto
FROM Transacao Trn
INNER JOIN ExtratoContaCorrente ECC ON ECC.ExtratoContaCorrenteTransacaoId = Trn.TransacaoId
INNER JOIN ContaCorrente CC ON CC.ContaCorrenteId = ECC.ExtratoContaCorrenteContaCorrenteId
INNER JOIN Participante Par ON Par.ParticipanteId = ECC.ExtratoContaCorrenteParticipanteId
WHERE 1=1
AND Trn.TransacaoId = $1' , 
FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Trocar Senha
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'TROCAR_SENHA', TRUE, 
    'SELECT DISTINCT ''{{CABECALHO_CUPOM}}'' ||
''<e><ce>ALTERAcaO DE SENHA</ce></e>'' ||
Adicionar_Quebra_Linha(true) ||
completa_textos(''USUARIO: '', COALESCE(Usu.UsuarioApelido, '''')) ||
Adicionar_Quebra_Linha() ||
completa_textos(''MOTIVO: '', COALESCE(Sen.SenhaMotivoAlteracao, '''')) ||
''{{RODAPE_CUPOM}}''
AS Texto
FROM Transacao Trn
INNER JOIN Senha Sen ON Sen.TransacaoId = Trn.TransacaoId
INNER JOIN Usuario Usu ON Usu.UsuarioId = Trn.TransacaoUsuarioId
WHERE 1=1
AND Trn.TransacaoId = $1' , 
FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Devolucao
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'DEVOLUCAO_TRN', TRUE, 
    'SELECT DISTINCT ''{{CABECALHO_CUPOM}}'' ||
''<e><ce>DEVOLUcaO</ce></e>'' ||
Adicionar_Quebra_Linha(true) ||
''</linha_simples>'' ||
Adicionar_Quebra_Linha(true) ||
completa_textos(''CLIENTE: '', Par.ParticipanteDocumento) ||
Adicionar_Quebra_Linha() ||
''<ad>'' || Par.ParticipanteRazaoSocial || ''</ad>'' ||
Adicionar_Quebra_Linha(true) ||
completa_textos(''DEVOLUCAO DO DOCUMENTO: '', CanIte.CancelamentoItemDocumento::text) ||
Adicionar_Quebra_Linha() ||
completa_textos(''MOTIVO: '', Can.CancelamentoObservacao) ||
Adicionar_Quebra_Linha() ||
''<ce>'' || ''PRODUTOS'' || ''</ce>'' ||
Adicionar_Quebra_Linha(true) ||
''</linha_simples>'' ||
Adicionar_Quebra_Linha() ||
COALESCE((
	SELECT string_agg(completa_textos(Pro.ProdutoNome, CanIte.CancelamentoItemValor::text) || Adicionar_Quebra_Linha(), '''') 
	  FROM CancelamentoItem CanIte
	 INNER JOIN Produto Pro ON Pro.ProdutoId = CanIte.CancelamentoItemProdutoId
	 WHERE CanIte.CancelamentoId = Can.CancelamentoId
  ), '''') ||    
Adicionar_Quebra_Linha() ||
completa_textos_trio('''', ''VALOR:'', Trn.TransacaoValor::text) ||
''{{RODAPE_CUPOM}}'' ||
''{{DEVOLUCAO_2VIA}}''
AS Texto
FROM Transacao Trn
INNER JOIN Participante Par ON Par.ParticipanteId = Trn.TransacaoParticipanteId
INNER JOIN Cancelamento Can ON Can.CancelamentoTransacao2Id = Trn.TransacaoId
INNER JOIN CancelamentoItem CanIte ON CanIte.CancelamentoId = Can.CancelamentoId
WHERE 1=1
AND Trn.TransacaoId = $1' , 
FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Devolucao Com Vale-Compra (2º Via)
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'DEVOLUCAO_2VIA', TRUE, 
    'SELECT DISTINCT ''{{CABECALHO_CUPOM}}'' ||
completa_textos(''CLIENTE: '', COALESCE(Par.ParticipanteDocumento, '''')) ||
Adicionar_Quebra_Linha() ||
''<ad>'' || COALESCE(Par.ParticipanteRazaoSocial, '''') || ''</ad>''||
Adicionar_Quebra_Linha(true) ||
completa_textos(''DOCUMENTO: '', COALESCE(CanIte.CancelamentoItemDocumento, 0)::text) ||
Adicionar_Quebra_Linha() ||
''<ae>'' || ''Vale-compra no valor de R$ '' || COALESCE(Trn.TransacaoValor, 0) || '', referente a Devolucao.'' || ''</ae>'' || 
Adicionar_Quebra_Linha(true) ||
''<ae>'' || ''Valido ate o dia '' || 
CASE 
    WHEN CC.ContaCorrenteValidade > 0 THEN to_char(current_date + CC.ContaCorrenteValidade * interval ''1 day'', ''DD/MM/YYYY'')
    ELSE to_char(date_trunc(''month'', current_date) + interval ''2 month'' - interval ''1 day'', ''DD/MM/YYYY'')
END 
 || ''</ae>'' ||
Adicionar_Quebra_Linha(true) ||
completa_textos_trio('''', ''VALOR:'', COALESCE(Trn.TransacaoValor, 0)::text) ||
Adicionar_Quebra_Linha() ||
Adicionar_Quebra_Linha() ||
COALESCE(FinCCC.FinanceiroCCCCodigoBarrasCodABar, '''') ||
Adicionar_Quebra_Linha(True) ||
''<ce>'' || COALESCE(FinCCC.FinanceiroCCCCodigoBarras, '''') || ''</ce>'' ||
''{{RODAPE_CUPOM}}''
AS Texto
FROM Transacao Trn
INNER JOIN Participante Par ON Par.ParticipanteId = Trn.TransacaoParticipanteId
INNER JOIN Cancelamento Can ON Can.CancelamentoTransacao2Id = Trn.TransacaoId
INNER JOIN CancelamentoItem CanIte ON CanIte.CancelamentoId = Can.CancelamentoId
INNER JOIN Financeiro Fin ON Fin.FinanceiroTransacaoId = Trn.TransacaoId
INNER JOIN FinanceiroCCC FinCCC ON FinCCC.FinanceiroId = Fin.FinanceiroId
INNER JOIN ContaCorrente CC ON CC.ContaCorrenteId = FinCCC.ContaCorrenteId
WHERE 1=1
AND Trn.TransacaoId = $1' , 
FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Troco com Vale-Compra
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'TROCO_VALE_COMPRA', TRUE, 
    'SELECT DISTINCT ''{{CABECALHO_CUPOM}}'' ||
       ''<e><ce>Vale-Compra</ce></e>'' ||
       Adicionar_Quebra_Linha(true) ||
       ''</linha_simples>'' 	    ||
       Adicionar_Quebra_Linha(true)     ||
       completa_textos(''Tipo: '', ''Troco de Vale-Compra'') ||
       Adicionar_Quebra_Linha() ||
       completa_textos(''Beneficiado: '', Par.ParticipanteDocumento) ||
       Adicionar_Quebra_Linha()     ||
       completa_textos('''', Par.ParticipanteRazaoSocial) ||
       Adicionar_Quebra_Linha()     ||
       completa_textos(''Vale-Compra: '', Replace(FinCCC.FinanceiroCCCComplemento, ''VC:'', '''')) ||
       Adicionar_Quebra_Linha() ||
       completa_textos_trio('''', ''Valor:'', ABS(FinCCC.FinanceiroCCCValor)::text) ||
       Adicionar_Quebra_Linha() ||
       Adicionar_Quebra_Linha() ||
       COALESCE(FinCCC.FinanceiroCCCCodigoBarrasCodABar, '''') ||
       Adicionar_Quebra_Linha(True) ||
      ''<ce>''  || COALESCE(FinCCC.FinanceiroCCCCodigoBarras, '''') || ''</ce>'' ||
       Adicionar_Quebra_Linha(True) ||
       ''{{RODAPE_CUPOM}}''
AS Texto
FROM Transacao Trn
INNER JOIN Participante Par ON Par.ParticipanteId = Trn.TransacaoParticipanteId
INNER JOIN Financeiro Fin ON Fin.FinanceiroTransacaoId = Trn.TransacaoId
INNER JOIN FinanceiroCCC FinCCC ON FinCCC.FinanceiroId = Fin.FinanceiroId
WHERE 1=1
AND Trn.TransacaoId = $1
AND FinCCC.FinanceiroCCCCodigoBarras <> '''' ' , 
FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Ressarcimento Vale-Compra
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'RESSARCIMENTO_VALE', TRUE, 
    'SELECT DISTINCT ''{{CABECALHO_CUPOM}}'' ||
completa_textos(''OPERACAO: '', ''Ressarcimento de Vale-Compra'') ||
Adicionar_Quebra_Linha() ||
completa_textos(''DOCUMENTO: '', 
	CASE -- Se nao estiver formatado com pontos, formata
	WHEN POSITION(''.'' IN MovCai.MovimentoCaixaHistorico) = 0 THEN
            SUBSTRING(MovCai.MovimentoCaixaHistorico FROM 1 FOR 6)  || ''.'' || 
            SUBSTRING(MovCai.MovimentoCaixaHistorico FROM 7 FOR 6)  || ''.'' || 
            SUBSTRING(MovCai.MovimentoCaixaHistorico FROM 13 FOR 6) || ''.'' || 
            SUBSTRING(MovCai.MovimentoCaixaHistorico FROM 19 FOR 6)
        ELSE
            MovCai.MovimentoCaixaHistorico -- Se ja estiver formatado, exibir o texto original
	END) ||
Adicionar_Quebra_Linha() ||
Adicionar_Quebra_Linha() ||
completa_textos_trio('''', ''VALOR:'', COALESCE(Trn.TransacaoValor, 0)::text) ||
''{{RODAPE_CUPOM}}''
AS Texto
FROM Transacao Trn
INNER JOIN Participante Par ON Par.ParticipanteId = Trn.TransacaoParticipanteId
INNER JOIN MovimentoCaixa MovCai ON MovCai.TransacaoId = Trn.TransacaoId
WHERE 1=1
AND Trn.TransacaoId = $1' , 
FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Compra de Vale-Presente
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'COMPRA_VALE_PRESENTE', TRUE, 
    'SELECT ''{{CABECALHO_CUPOM}}'' ||
       ''<e><ce>Comprovante de Vale</ce></e>'' ||
       Adicionar_Quebra_Linha(true) ||
       ''</linha_simples>'' 	    ||
       Adicionar_Quebra_Linha(true)     ||
       completa_textos(''Tipo: '', CASE WHEN FinanceiroCCCTIpo = ''D'' THEN ''Compra de Vale-Presente'' ELSE ''Troco de Vale-Presente'' END) ||
       Adicionar_Quebra_Linha() ||
       completa_textos(''Beneficiado: '', Trn.TransacaoObservacao) ||
       Adicionar_Quebra_Linha()     ||
       completa_textos(''Vale-Presente: '', Replace(FinCCC.FinanceiroCCCComplemento, ''VP:'', '''')) ||
       Adicionar_Quebra_Linha() ||
       completa_textos_trio('''', ''Valor:'', ABS(FinCCC.FinanceiroCCCValor)::text) ||
       Adicionar_Quebra_Linha() ||
       Adicionar_Quebra_Linha() ||
       COALESCE(FinCCC.FinanceiroCCCCodigoBarrasCodABar, '''') || 
       Adicionar_Quebra_Linha(True) ||
      ''<ce>''  || COALESCE(FinCCC.FinanceiroCCCCodigoBarras, '''') || ''</ce>'' ||
       Adicionar_Quebra_Linha(True) ||
       ''{{RODAPE_CUPOM}}''
   AS Texto
 FROM Transacao Trn
INNER JOIN Financeiro Fin ON Fin.FinanceiroTransacaoId = Trn.TransacaoId
INNER JOIN FinanceiroCCC FinCCC ON FinCCC.FinanceiroId = Fin.FinanceiroId
 WHERE 1=1
  AND Trn.TransacaoId = $1' , 
FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Comprovante Gerar Vale-Troca
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'GERAR_VALE_TROCA', TRUE, 
    'SELECT ''{{CABECALHO_CUPOM}}'' ||
       ''<e><ce>Cupom de Troca</ce></e>'' ||
       Adicionar_Quebra_Linha(true) ||
       ''</linha_simples>'' 	    ||
       Adicionar_Quebra_Linha(true) ||
       completa_textos(''Beneficiado: '', Par.ParticipanteRazaoSocial) ||
       Adicionar_Quebra_Linha() ||
       completa_textos_trio('''', ''Valor:'', Trn.TransacaoValor::text) ||
       Adicionar_Quebra_Linha() ||
       Adicionar_Quebra_Linha() ||
      ''<ce>'' || COALESCE(CupTro.CupomTrocaCodigoBarras, '''') || ''</ce>'' ||
      ''{{RODAPE_CUPOM}}''
   AS Texto
 FROM Transacao Trn
INNER JOIN CupomTroca CupTro ON CupTro.CupomTrocaTransacaoId = Trn.TransacaoId
INNER JOIN Documento Doc ON Doc.DocumentoId = CupTro.CupomTrocaDocumentoId
INNER JOIN Transacao Trn2 ON Doc.DocumentoTransacaoId = Trn2.TransacaoId
INNER JOIN Participante Par ON Par.ParticipanteId = Trn2.TransacaoParticipanteId
 WHERE 1=1
  AND Trn.TransacaoId = $1' , 
FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Observacao Fiscal
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'OBS_FISCAL', TRUE, 
    'SELECT REPLACE(REPLACE(REPLACE(REPLACE( CAST(array_agg(tab.TXTOBS) AS VARCHAR(4000)) ,''"'', '''') ,''{'', '''') ,''}'', '''') ,''>,'', ''>'') FROM ( 
      SELECT
      CASE WHEN substr(DocumentoItem.DocumentoItemCSTICMS, 2,2)=''20''AND DocumentoItem.DocumentoItemAliquotaReducao = 48.24 
               THEN ''BASE DE CALC. RED. CFE. LIVRO I, ART 23, INC XIII, DEC 37.699/97 - RICMS/RS.''
          WHEN substr(DocumentoItem.DocumentoItemCSTICMS, 2,2)=''20''AND DocumentoItem.DocumentoItemAliquotaReducao = 80.00
               THEN ''BASE DE CALC. RED. CFE. LIVRO I, ART 23, INC I, DEC 37.699/97 - RICMS/RS.''
           WHEN substr(DocumentoItem.DocumentoItemCSTICMS, 2,2)=''20''AND DocumentoItem.DocumentoItemAliquotaReducao = 41.67
               THEN ''BASE DE CALC. RED. CFE. LIVRO I, ART 23, INC I, DEC 37.699/97 - RICMS/RS.''
          WHEN substr(DocumentoItem.DocumentoItemCSTICMS, 2,2)=''20''AND DocumentoItem.DocumentoItemAliquotaReducao = 58.82
               THEN ''BASE DE CALC. RED. CFE. LIVRO I, ART 23, INC II E III, DEC 37.699/97 - RICMS/RS.''
          WHEN substr(DocumentoItem.DocumentoItemCSTICMS, 2,2)=''90'' AND DocumentoItem.DocumentoItemNaturezaFiscal in (''592200'',''692200'')
               THEN ''Incluso em OUTRAS TAXAS R$ '' || sum(DocumentoItem.DocumentoItemValorImpostosFatura) || '' referente ao ICMS por substituicao tributaria a ser destacado nas notas fiscais de REMESSA posteriores (CFOP 5/6.116 ou 5/6.117)'' 
          WHEN substr(DocumentoItem.DocumentoItemCSTICMS, 2,2)=''60'' 
               THEN ''IMPOSTO RETIDO ANTERIORMENTE P/SBUST. TRIBUTaRIA CFE. PROT. ICMS DE 23 DE JULHO DE 2009.''
           ELSE ''''
      END AS TXTOBS
      FROM Transacao INNER JOIN Documento ON Transacao.TransacaoId = Documento.DocumentoTransacaoId
      INNER JOIN DocumentoItem ON DocumentoItem.DocumentoId = Documento.DocumentoId
      WHERE Transacao.TransacaoId = 3970 
      AND DocumentoItem.DocumentoItemSituacao <> ''C''
      GROUP BY DocumentoItem.DocumentoItemCSTICMS, DocumentoItem.DocumentoItemAliquotaReducao, DocumentoItem.DocumentoItemCodigoBeneficioFiscal, DocumentoItem.DocumentoItemNaturezaFiscal
      ) TAB;' , 
FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Itens Retira no Depósito
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'ITEM_RETIRA_DEPOSITO', TRUE, 
    'SELECT ''{{CABECALHO_CUPOM}}'' ||
	''<ce>'' || ''RETIRAR NO DEPOSITO'' || ''</ce>'' ||
	Adicionar_Quebra_Linha(true) ||
	''</linha_simples>'' ||
	Adicionar_Quebra_Linha(true) ||	
   completa_textos(''Cliente:'',  
      CASE WHEN (Cli.ParticipanteDocumento <> '''')
   		THEN (
               CASE 
                 WHEN LENGTH(Cli.ParticipanteDocumento) = 11
                 THEN Trim(replace(to_char(To_Number(Cli.ParticipanteDocumento,''99999999999''), ''000:000:000-00''), '':'', ''.''))
                   ELSE Trim(replace(to_char(To_Number(Cli.ParticipanteDocumento,''9999999999999''), ''00:000:000/0000-00''), '':'', ''.''))
                 END
              )
         ELSE '''' END) ||
   Adicionar_Quebra_Linha() ||
   completa_textos('''', Cli.ParticipanteRazaoSocial || '' '' || Cli.ParticipanteNomeFantasia) ||
   Adicionar_Quebra_Linha() ||
   completa_textos(''Vendedor:'', Ven.UsuarioApelido) ||
   Adicionar_Quebra_Linha() ||
   Adicionar_Quebra_Linha() ||
	completa_textos_trio_edentado(''Cod. Referencia'', ''Desc.'', ''Qtde'') ||
        Adicionar_Quebra_Linha() ||
        completa_textos_trio_edentado(''---------------'', ''-----'', ''----'') ||
        Adicionar_Quebra_Linha() ||
        COALESCE((
		SELECT 
			string_agg(completa_textos_trio_edentado(Pro.ProdutoReferencia, '''', Trunc(DocIte.DocumentoItemQuantidade,1)::text) || Adicionar_Quebra_Linha() || completa_textos_trio_edentado(Pro.ProdutoNome, '''', '''') || Adicionar_Quebra_Linha(), '''')
		  FROM Documento Doc
		 INNER JOIN DocumentoItem DocIte ON DocIte.DocumentoId = Doc.DocumentoId
		 INNER JOIN Produto Pro ON Pro.ProdutoId = DocIte.DocumentoItemProdutoId
		 WHERE Doc.DocumentoTransacaoId = Trn.TransacaoId
		   AND DocIte.DocumentoItemTipoEntrega in (''3'')
		  ), '''') ||
	Adicionar_Quebra_Linha() ||
	completa_textos_trio_edentado('''', ''Qtd. Total'', 
	COALESCE((SELECT Trunc(SUM(DocumentoItemQuantidade),1)::text
	   FROM Documento Doc
	  INNER JOIN DocumentoItem DocIte ON DocIte.DocumentoId = Doc.DocumentoId
	  INNER JOIN Produto Pro ON Pro.ProdutoId = DocIte.DocumentoItemProdutoId
	  WHERE Doc.DocumentoTransacaoId = Trn.TransacaoId
	    AND DocIte.DocumentoItemTipoEntrega in (''3'')), '''')
	 ) ||
       ''{{RODAPE_CUPOM}}''
	AS Texto
 FROM Transacao Trn
INNER JOIN Participante Cli ON Cli.ParticipanteId = Trn.TransacaoParticipanteId
INNER JOIN Documento Doc ON Doc.DocumentoTransacaoId = Trn.TransacaoId
INNER JOIN Usuario Ven ON Ven.UsuarioId = Doc.DocumentoVendedorId
WHERE Trn.TransacaoId = $1' , 
FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Total por funcoes
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'TOTAL_FUNCOES', TRUE, 
    'SELECT 
    ''</linha_simples>'' ||
	 Adicionar_Quebra_Linha(true) ||	
    ''<ce>''  || ''TOTAL POR FUNCAO'' || ''</ce>'' ||
    Adicionar_Quebra_Linha(true) ||
    ''</linha_simples>'' ||
	 Adicionar_Quebra_Linha(true) ||	
      completa_textos_trio_edentado(''FUNCAO'', ''TIPO'', ''VALOR'') ||  
       Adicionar_Quebra_Linha() ||
       completa_textos_trio_edentado(''------'', ''----'', ''-----'') ||   
       Adicionar_Quebra_Linha() ||  
       COALESCE((
         SELECT 
            string_agg(completa_textos_trio_edentado(coalesce(Funcao, ''''), coalesce(Tipo, '''') ,CAST(Valor AS VARCHAR)) || Adicionar_Quebra_Linha(), '''')  
        FROM (
            SELECT 
                (CASE
			       WHEN Trn1.TransacaoTipo = 99 AND Trn1.TransacaoTipoVenda = 2 THEN case WHEN Nat.NaturezaFuncao IS NOT NULL THEN Nat.naturezafuncao::varchar else (Nat.naturezacodigo + 500)::varchar END -- Venda em NFe
			        WHEN Trn1.TransacaoTipo IN (799, 877) THEN case when (sanref.sangriareforcofuncao is not null) and (sanref.sangriareforcofuncao <> 0) then SanRef.sangriareforcofuncao::varchar else (SanRef.sangriareforcocodigomenu + 700)::varchar END	-- Sangria ou Ressarcimento de Vale
			        ELSE Trn1.TransacaoTipo::varchar 
			    END) AS Funcao,
                (CASE
			        WHEN Trn1.TransacaoTipo = 99 AND Trn1.TransacaoTipoVenda = 2 THEN coalesce(Nat.naturezadescricao, ''NFe'') -- Venda em NFe
			        WHEN Trn1.TransacaoTipo = 99 AND Trn1.TransacaoTipoVenda = 3 THEN ''NFCe'' -- Venda em NFCe
			        WHEN Trn1.TransacaoTipo = 99 AND Trn1.TransacaoTipoVenda = 5 THEN ''SAT'' -- Venda em SAT
			        WHEN Trn1.TransacaoTipo IN (799, 877) THEN SanRef.sangriareforconome -- Sangria ou Ressarcimento de Vale
			        ELSE 
			            CASE
			                WHEN Trn1.TransacaoTipo = 1 THEN ''Manutencao e suporte''
			                WHEN Trn1.TransacaoTipo = 2 THEN ''Carga''
			                WHEN Trn1.TransacaoTipo = 3 THEN ''Carga inicial''
			                WHEN Trn1.TransacaoTipo = 10 THEN ''Vazia''
			                WHEN Trn1.TransacaoTipo = 50 THEN ''Pre-Venda''
			                WHEN Trn1.TransacaoTipo = 51 THEN ''Cliente via Pre-Venda''
			                WHEN Trn1.TransacaoTipo = 52 THEN ''Solicita credito''
			                WHEN Trn1.TransacaoTipo = 59 THEN ''Cancela Pre-Venda''
			                WHEN Trn1.TransacaoTipo = 60 THEN ''O.S. (Ordem de Servico)''
			                WHEN Trn1.TransacaoTipo = 61 THEN ''Status de O.S.''
			                WHEN Trn1.TransacaoTipo = 62 THEN ''Reimpressao de O.S.''
			                WHEN Trn1.TransacaoTipo = 63 THEN ''Protocolo NFC-e''
			                WHEN Trn1.TransacaoTipo = 64 THEN ''Protocolo Cancelamento''
			                WHEN Trn1.TransacaoTipo = 99 THEN ''Venda''
			                WHEN Trn1.TransacaoTipo = 100 THEN ''Abre Dia''
			                WHEN Trn1.TransacaoTipo = 101 THEN ''Abre Caixa''
			                WHEN Trn1.TransacaoTipo = 102 THEN ''Cupom Fiscal''
			                WHEN Trn1.TransacaoTipo = 103 THEN ''Nota Fiscal''
			                WHEN Trn1.TransacaoTipo = 104 THEN ''Venda (NFCe)''
			                WHEN Trn1.TransacaoTipo = 105 THEN ''Venda (SAT)''
			                WHEN Trn1.TransacaoTipo = 106 THEN ''Recibo de Venda''
			                WHEN Trn1.TransacaoTipo = 107 THEN ''Registro de Nota Manual''
			                WHEN Trn1.TransacaoTipo = 109 THEN ''RPS (Recibo Provisorio de Servicos)''
			                WHEN Trn1.TransacaoTipo = 200 THEN ''Bloqueia o caixa (Shift F10)''
			                WHEN Trn1.TransacaoTipo = 201 THEN ''Propaganda com bloqueio de caixa  (Shift F11)''
			                WHEN Trn1.TransacaoTipo = 202 THEN ''Propaganda sem bloqueio de caixa  (Shift F12)''
			                WHEN Trn1.TransacaoTipo = 203 THEN ''Retirada de cheques''
			                WHEN Trn1.TransacaoTipo = 204 THEN ''Abre gaveta''
			                WHEN Trn1.TransacaoTipo = 290 THEN ''Re-imprime caixa''
			                WHEN Trn1.TransacaoTipo = 299 THEN ''Fecha caixa''
			                WHEN Trn1.TransacaoTipo = 300 THEN ''Estorno de transacoes''
			                WHEN Trn1.TransacaoTipo = 301 THEN ''Devolucao (consulta documento)''
			                WHEN Trn1.TransacaoTipo = 302 THEN ''Devolucao (por TAG ou RFID)''
			                WHEN Trn1.TransacaoTipo = 303 THEN ''Ativacao de Vale Presente (via TEF)''
			                WHEN Trn1.TransacaoTipo = 304 THEN ''Recarga de Vale Presente (via TEF)''
			                WHEN Trn1.TransacaoTipo = 305 THEN ''Consulta de Vale Presente (via TEF)''
			                WHEN Trn1.TransacaoTipo = 306 THEN ''Servicos adicionais (Seguros, garantia estendida ...)''
			                WHEN Trn1.TransacaoTipo = 307 THEN ''Vale gas''
			                WHEN Trn1.TransacaoTipo = 308 THEN ''Vale Presente (proprio)''
			                WHEN Trn1.TransacaoTipo = 309 THEN ''Consulta Vale Presente (proprio)''
			                WHEN Trn1.TransacaoTipo = 310 THEN ''Cupom de Troca''
			                WHEN Trn1.TransacaoTipo = 311 THEN ''Fatura consignados''
			                WHEN Trn1.TransacaoTipo = 312 THEN ''Extrato de consignacao''
			                WHEN Trn1.TransacaoTipo = 313 THEN ''Seleciona tabela de precos''
			                WHEN Trn1.TransacaoTipo = 314 THEN ''Recarga de celular''
			                WHEN Trn1.TransacaoTipo = 380 THEN ''Copia itens de venda''
			                WHEN Trn1.TransacaoTipo = 390 THEN ''Cancelamento de Cupom Fiscal''
			                WHEN Trn1.TransacaoTipo = 399 THEN ''O.S. (Ordem de Servico)''
			                WHEN Trn1.TransacaoTipo = 400 THEN ''Recebimento de debitos de clientes''
			                WHEN Trn1.TransacaoTipo = 401 THEN ''Recebimento de contas externas (Titulos, agua, luz, telefone... )''
			                WHEN Trn1.TransacaoTipo = 402 THEN ''Deposito em conta corrente''
			                WHEN Trn1.TransacaoTipo = 403 THEN ''Retirada de conta corrente''
			                WHEN Trn1.TransacaoTipo = 404 THEN ''Extrato de conta corrente''
			                WHEN Trn1.TransacaoTipo = 405 THEN ''Recebimento de pendencias''
			                WHEN Trn1.TransacaoTipo = 406 THEN ''Extrato de bonus''
			                WHEN Trn1.TransacaoTipo = 407 THEN ''Resgate de bonus''
			                WHEN Trn1.TransacaoTipo = 408 THEN ''Extrato de Debitos''
			                WHEN Trn1.TransacaoTipo = 600 THEN ''LX (Leitura X)''
			                WHEN Trn1.TransacaoTipo = 601 THEN ''LMF (Leitura de Memoria Fiscal)''
			                WHEN Trn1.TransacaoTipo = 602 THEN ''Arq. MF (Gera arquivo de Memoria Fiscal)''
			                WHEN Trn1.TransacaoTipo = 603 THEN ''Arq. MFD (Gera arquivo de Memoria Fiscal Detalhada)''
			                WHEN Trn1.TransacaoTipo = 604 THEN ''Identificacao do PAF-NFC-e''
			                WHEN Trn1.TransacaoTipo = 605 THEN ''Tab. indice Tecnico Producao''
			                WHEN Trn1.TransacaoTipo = 606 THEN ''Parametros de configuracao''
			                WHEN Trn1.TransacaoTipo = 607 THEN ''Registros do PAF-NFC-e''
			                WHEN Trn1.TransacaoTipo = 608 THEN ''Espelho MFD''
			                WHEN Trn1.TransacaoTipo = 609 THEN ''Envio ao FISCO - REDUCAO Z''
			                WHEN Trn1.TransacaoTipo = 610 THEN ''Envio ao FISCO - ESTOQUE''
			                WHEN Trn1.TransacaoTipo = 611 THEN ''Vendas Identificadas CPF ou CNPJ''
			                WHEN Trn1.TransacaoTipo = 612 THEN ''Requisicoes Externas Registradas''
			                WHEN Trn1.TransacaoTipo = 700 THEN ''Reforco''
			                WHEN Trn1.TransacaoTipo = 799 THEN ''Sangria''
			                WHEN Trn1.TransacaoTipo = 800 THEN ''Cadastro de clientes''
			                WHEN Trn1.TransacaoTipo = 801 THEN ''Impressao de DAV (selecao)''
			                WHEN Trn1.TransacaoTipo = 802 THEN ''Impressao de DAV (abertas)''
			                WHEN Trn1.TransacaoTipo = 803 THEN ''Troca de senha''
			                WHEN Trn1.TransacaoTipo = 804 THEN ''Pendencias de TEF''
			                WHEN Trn1.TransacaoTipo = 805 THEN ''Reimpressao e envio de e-mails (Documentos fiscais da empresa)''
			                WHEN Trn1.TransacaoTipo = 806 THEN ''Reimpressao e envio de e-mails (transacoes deste terminal)''
			                WHEN Trn1.TransacaoTipo = 807 THEN ''Opcoes administrativas de TEF''
			                WHEN Trn1.TransacaoTipo = 808 THEN ''Lancamento de Estoque''
			                WHEN Trn1.TransacaoTipo = 809 THEN ''Reinicia leitora de RFID''
			                WHEN Trn1.TransacaoTipo = 810 THEN ''Mescla de DAVs''
			                WHEN Trn1.TransacaoTipo = 811 THEN ''Lancamentos do caixa''
			                WHEN Trn1.TransacaoTipo = 812 THEN ''Acesso de Supervisor''
			                WHEN Trn1.TransacaoTipo = 813 THEN ''Saida do Modo Supervisor''
			                WHEN Trn1.TransacaoTipo = 850 THEN ''Carga de tabelas do PinPad (Forcada)''
			                WHEN Trn1.TransacaoTipo = 877 THEN ''Ressarcimento de Vale-Compra''
			                WHEN Trn1.TransacaoTipo = 880 THEN ''Alertas do sistema (Shift F4)''
			                WHEN Trn1.TransacaoTipo = 881 THEN ''Status da comunicacao com o Sefaz (Shift F5)''
			                WHEN Trn1.TransacaoTipo = 882 THEN ''Status da comunicacao com o servidor (Shift F6)''
			                WHEN Trn1.TransacaoTipo = 883 THEN ''Status da comunicacao com o TEF (Shift F7)''
			                WHEN Trn1.TransacaoTipo = 890 THEN ''Controle de transmissao''
			                WHEN Trn1.TransacaoTipo = 891 THEN ''Configuracoes de ambiente''
			                WHEN Trn1.TransacaoTipo = 892 THEN ''Configuracoes de formulas''
			                WHEN Trn1.TransacaoTipo = 893 THEN ''Exportacao cadastral (Dados para outros terminais)''
			                WHEN Trn1.TransacaoTipo = 894 THEN ''Sincronismo manual com servidor''
			                WHEN Trn1.TransacaoTipo = 899 THEN ''Relatorio de configuracao''
			                WHEN Trn1.TransacaoTipo = 995 THEN ''Reimpressao em NF-e''
			                WHEN Trn1.TransacaoTipo = 996 THEN ''e-Mail Boleto''
			                WHEN Trn1.TransacaoTipo = 997 THEN ''Envio de e-Mail da NFC-e''
			                WHEN Trn1.TransacaoTipo = 998 THEN ''Carga de tabelas do PinPad''
			                WHEN Trn1.TransacaoTipo = 999 THEN ''Fecha dia''
			                ELSE ''Outro tipo de transacao''
			            END
			    end) AS Tipo,
                SUM(Fin.FinanceiroValorPrimeira + Fin.FinanceiroValorEntrada + (Fin.FinanceiroValorParcelas * Fin.FinanceiroQuantidadeParcelas) - Fin.FinanceiroValorTroco) AS Valor 
            FROM Transacao Trn1
           INNER JOIN Financeiro Fin ON Fin.FinanceiroTransacaoId = Trn1.TransacaoId
           left join Documento Doc on Doc.DocumentoTransacaoId = Trn1.TransacaoId
		   left join Natureza Nat on Nat.naturezaid = Doc.naturezaid 
		   left join MovimentoCaixa MovCai on MovCai.transacaoid  = Trn1.TransacaoId
		   left join SangriaReforco SanRef on SanRef.sangriareforcoid  = MovCai.sangriareforcoid 
           WHERE Trn1.TransacaoCaixaId = Trn.TransacaoCaixaId  
             AND Trn1.TransacaoTipo <> 299 -- Fechar Caixa
             AND Fin.FinanceiroSituacao <> ''C''
            GROUP BY Trn1.TransacaoTipo, Trn1.TransacaoTipoVenda, Nat.NaturezaFuncao, Nat.NaturezaCodigo, SanRef.SangriaReforcoFuncao, SanRef.SangriaReforcoCodigoMenu, Nat.NaturezaDescricao, SanRef.SangriaReforcoNome
        ) Subquery
    ), '''''''') AS Texto
FROM Transacao Trn 
INNER JOIN Caixa Cai ON Cai.CaixaId = Trn.TransacaoCaixaId
INNER JOIN Dia Dia ON Dia.DiaId = Cai.CaixaDiaId
INNER JOIN Transacao Abertura ON Abertura.TransacaoId = Dia.DiaAberturaId
WHERE Trn.TransacaoId = $1
', FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();

-- Total Lançamentos Financeiros LANCAMENTOS_FIN
INSERT INTO public.sql (
    sqlchave, sqlativo, sqlexpressao, sqlemvalidacao, sqlvalidacao, 
    sqlpadrao, sqlinclusaodatahora, sqlalteracaodatahora, sqlassinatura
) VALUES (
    'LANCAMENTOS_FIN', TRUE, 
    'SELECT 
    ''</linha_simples>'' ||
	 Adicionar_Quebra_Linha(true) ||	
    ''<ce>''  || ''LANCAMENTOS'' || ''</ce>'' ||
    Adicionar_Quebra_Linha(true) ||
    ''</linha_simples>'' ||
	 Adicionar_Quebra_Linha(true) ||	
      completa_textos_trio_edentado(''HORA     MEIO'', ''DOCUMENTO   FUN'', ''VALOR'') ||  
       Adicionar_Quebra_Linha() ||
       completa_textos_trio_edentado(''-------- ----'', ''---------   ---'', ''-----'') ||   
       Adicionar_Quebra_Linha() ||  
       COALESCE((
         SELECT 
            string_agg(completa_textos_trio_edentado(Substr(DataHora::varchar, 12, 8) || '' '' || Meio, 
                                                     CASE WHEN Documento::varchar = ''0'' THEN ''---------'' ELSE LPAD(Documento::varchar, 9, ''0'') End || ''   '' || LPAD(Funcao, 3, '' ''),
                                                     CAST(Valor AS VARCHAR)) || 
                                                     Adicionar_Quebra_Linha(), '''')  
        FROM (
            SELECT 
                Trn1.TransacaoInicio AS DataHora,
                Coalesce(Doc1.documentonumero, 0) AS Documento,
                Mei.MeioDescricao as Meio,
                CASE
                  WHEN Trn1.TransacaoTipo = 99 AND Trn1.TransacaoTipoVenda = 2 THEN case WHEN Nat.NaturezaFuncao IS NOT NULL THEN Nat.naturezafuncao::varchar else (Nat.naturezacodigo + 500)::varchar END -- Venda em NFe
                  WHEN Trn1.TransacaoTipo IN (799, 877) THEN case when (sanref.sangriareforcofuncao is not null) and (sanref.sangriareforcofuncao <> 0) then SanRef.sangriareforcofuncao::varchar else (SanRef.sangriareforcocodigomenu + 700)::varchar END	-- Sangria ou Ressarcimento de Vale
                  ELSE Trn1.TransacaoTipo::varchar 
                END AS Funcao,
                SUM(Fin.FinanceiroValorPrimeira + Fin.FinanceiroValorEntrada + (Fin.FinanceiroValorParcelas * Fin.FinanceiroQuantidadeParcelas) - Fin.FinanceiroValorTroco) AS Valor 
            FROM Transacao Trn1
           INNER JOIN Financeiro Fin ON Fin.FinanceiroTransacaoId = Trn1.TransacaoId
           inner join Meio Mei on Mei.MeioId = Fin.FinanceiroMeioId
           left  join Documento Doc1 on Doc1.documentotransacaoid  = Trn1.TransacaoId
           left  join Natureza Nat on Nat.naturezaid = Doc1.naturezaid 
           left  join MovimentoCaixa MovCai on MovCai.transacaoid  = Trn1.TransacaoId
           left  join SangriaReforco SanRef on SanRef.sangriareforcoid  = MovCai.sangriareforcoid 
           WHERE Trn1.TransacaoCaixaId = Trn.TransacaoCaixaId  
             AND Trn1.TransacaoTipo <> 299 -- Fechar Caixa
             AND Fin.FinanceiroSituacao <> ''C''
            GROUP BY Trn1.TransacaoTipo, Trn1.TransacaoTipoVenda, Trn1.TransacaoInicio, Doc1.DocumentoNumero, Mei.MeioDescricao, Nat.NaturezaFuncao, Nat.NaturezaCodigo, SanRef.SangriaReforcoFuncao, SanRef.SangriaReforcoCodigoMenu, Nat.NaturezaDescricao, SanRef.SangriaReforcoNome
            ORDER BY Trn1.transacaoinicio 
        ) Subquery 
    ), '''''''')  || 
    Adicionar_Quebra_Linha() || 
    COALESCE((
         SELECT 
            string_agg(completa_textos_trio_edentado('''', ''TOTAL'' , CAST(Valor AS VARCHAR)) || Adicionar_Quebra_Linha(), '''')  
        FROM (
            SELECT 
                SUM(Fin.FinanceiroValorPrimeira + Fin.FinanceiroValorEntrada + (Fin.FinanceiroValorParcelas * Fin.FinanceiroQuantidadeParcelas) - Fin.FinanceiroValorTroco) AS Valor 
            FROM Transacao Trn1
           INNER JOIN Financeiro Fin ON Fin.FinanceiroTransacaoId = Trn1.TransacaoId
           WHERE Trn1.TransacaoCaixaId = Trn.TransacaoCaixaId  
             AND Trn1.TransacaoTipo <> 299 -- Fechar Caixa
             AND Fin.FinanceiroSituacao <> ''C''
        ) Subquery 
    ), '''''''')  AS Texto
FROM Transacao Trn 
WHERE Trn.TransacaoId = $1
', FALSE, '', 
    TRUE, now(), now(), ''
) ON CONFLICT (sqlchave) 
DO UPDATE SET 
sqlexpressao = excluded.sqlexpressao,
sqlemvalidacao = excluded.sqlemvalidacao,
sqlativo = excluded.sqlativo,
sqlvalidacao = excluded.sqlvalidacao,
sqlpadrao = excluded.sqlpadrao,
sqlinclusaodatahora = now(),
sqlalteracaodatahora = now();